@@ -5,10 +5,18 @@ PLATFORM=OS
 VERBOSE=no
 DEBUG=no
 SDK_VERSION=`xcrun --sdk iphoneos --show-sdk-version`
-SDK_MIN=6.1
+SDK_MIN=7.0
 SIXTYFOURBIT_SDK_MIN=7.0
 ARCH=armv7
 SCARY=yes
+TVOS=no
+OSSTYLE=iPhone
+OSVERSIONMINCFLAG=miphoneos-version-min
+OSVERSIONMINLDFLAG=ios_version_min
+BITCODE=no
+
+CORE_COUNT=`sysctl -n machdep.cpu.core_count`
+let MAKE_JOBS=$CORE_COUNT+1
 
 usage()
 {
@@ -21,7 +29,9 @@ OPTIONS
    -a <arch>     Specify which arch to use (current: ${ARCH})
    -d            Enable debug
    -v            Enable verbose command-line output
+   -t            Build for tvOS
    -w            Build a limited stack of non-scary libraries only
+   -b            Enable bitcode
 EOF
 }
 
@@ -42,7 +52,7 @@ info()
     echo "[${blue}info${normal}] $1"
 }
 
-while getopts "hvwdsk:a:" OPTION
+while getopts "hvbwdstk:a:" OPTION
 do
      case $OPTION in
          h)
@@ -61,12 +71,23 @@ do
          w)
              SCARY=no
              ;;
+         b)
+             BITCODE=yes
+             ;;
          k)
              SDK_VERSION=$OPTARG
              ;;
          a)
              ARCH=$OPTARG
              ;;
+         t)
+             TVOS=yes
+             BITCODE=yes
+             SDK_VERSION=`xcrun --sdk appletvos --show-sdk-version`
+             OSVERSIONMINCFLAG=mtvos-version-min
+             OSVERSIONMINLDFLAG=tvos_version_min
+             SIXTYFOURBIT_SDK_MIN=9.0
+             ;;
          ?)
              usage
              exit 1
@@ -85,12 +106,14 @@ if [ "$VERBOSE" = "yes" ]; then
    out="/dev/stdout"
 fi
 
-info "Building libvlc for iOS"
+TARGET="${ARCH}-apple-darwin11"
 
-if [ "$PLATFORM" = "Simulator" ]; then
-    TARGET="${ARCH}-apple-darwin11"
+# apple doesn't call AArch64 that way, but arm64 (a contrario to all libraries)
+# so we need to translate it..
+if [ "$ARCH" = "aarch64" ]; then
+	ACTUAL_ARCH="arm64"
 else
-    TARGET="arm-apple-darwin11"
+	ACTUAL_ARCH="$ARCH"
 fi
 
 if [ "$DEBUG" = "yes" ]; then
@@ -99,6 +122,14 @@ else
     OPTIM="-O3 -g"
 fi
 
+if [ "$TVOS" = "yes" ]; then
+	OSSTYLE=AppleTV
+	export BUILDFORTVOS="yes"
+fi
+export BUILDFORIOS="yes"
+
+info "Building libvlc for Apple embedded OS style '${OSSTYLE}'"
+
 info "Using ${ARCH} with SDK version ${SDK_VERSION}"
 
 THIS_SCRIPT_PATH=`pwd`/$0
@@ -109,8 +140,8 @@ spopd
 
 if test -z "$SDKROOT"
 then
-    SDKROOT=`xcode-select -print-path`/Platforms/iPhone${PLATFORM}.platform/Developer/SDKs/iPhone${PLATFORM}${SDK_VERSION}.sdk
-    echo "SDKROOT not specified, assuming $SDKROOT"
+	SDKROOT=`xcode-select -print-path`/Platforms/${OSSTYLE}${PLATFORM}.platform/Developer/SDKs/${OSSTYLE}${PLATFORM}${SDK_VERSION}.sdk
+	echo "SDKROOT not specified, assuming $SDKROOT"
 fi
 
 if [ ! -d "${SDKROOT}" ]
@@ -119,9 +150,8 @@ then
     exit 1
 fi
 
-BUILDDIR="${VLCROOT}/build-ios-${PLATFORM}/${ARCH}"
-
-PREFIX="${VLCROOT}/install-ios-${PLATFORM}/${ARCH}"
+BUILDDIR="${VLCROOT}/build-ios-${OSSTYLE}${PLATFORM}/${ACTUAL_ARCH}"
+PREFIX="${VLCROOT}/install-ios-${OSSTYLE}${PLATFORM}/${ACTUAL_ARCH}"
 
 export PATH="${VLCROOT}/extras/tools/build/bin:${VLCROOT}/contrib/${TARGET}/bin:/usr/bin:/bin:/usr/sbin:/sbin:/usr/local/bin:/usr/X11/bin"
 
@@ -131,7 +161,7 @@ spushd "${VLCROOT}/extras/tools"
 make && make .gas
 spopd
 
-info "Building contrib for iOS in '${VLCROOT}/contrib/iPhone${PLATFORM}-${ARCH}'"
+info "Building contrib for iOS in '${VLCROOT}/contrib/${OSSTYLE}${PLATFORM}-${ARCH}'"
 
 # The contrib will read the following
 export AR="xcrun ar"
@@ -145,53 +175,56 @@ export STRIP="xcrun strip"
 
 export PLATFORM=$PLATFORM
 export SDK_VERSION=$SDK_VERSION
+export VLCSDKROOT=$SDKROOT
 
-export CFLAGS="-isysroot ${SDKROOT} -arch ${ARCH} ${OPTIM}"
+CFLAGS="-isysroot ${SDKROOT} -arch ${ACTUAL_ARCH} ${OPTIM}"
 
 if [ "$PLATFORM" = "OS" ]; then
-if [ "$ARCH" != "arm64" ]; then
-export CFLAGS="${CFLAGS} -mcpu=cortex-a8 -miphoneos-version-min=${SDK_MIN}"
+if [ "$ARCH" != "aarch64" ]; then
+CFLAGS+=" -mcpu=cortex-a8 -${OSVERSIONMINCFLAG}=${SDK_MIN}"
 else
-export CFLAGS="${CFLAGS} -miphoneos-version-min=${SIXTYFOURBIT_SDK_MIN}"
+CFLAGS+=" -${OSVERSIONMINCFLAG}=${SIXTYFOURBIT_SDK_MIN}"
 fi
 else
-export CFLAGS="${CFLAGS} -miphoneos-version-min=${SIXTYFOURBIT_SDK_MIN}"
+CFLAGS+=" -${OSVERSIONMINCFLAG}=${SIXTYFOURBIT_SDK_MIN}"
 fi
 
-export CXXFLAGS="${CFLAGS} -stdlib=libstdc++"
+if [ "$BITCODE" = "yes" ]; then
+CFLAGS+=" -fembed-bitcode"
+fi
 
+export CFLAGS="${CFLAGS}"
+export CXXFLAGS="${CFLAGS}"
 export CPPFLAGS="${CFLAGS}"
 
 export CPP="xcrun cc -E"
 export CXXCPP="xcrun c++ -E"
 
-export BUILDFORIOS="yes"
-
 if [ "$PLATFORM" = "Simulator" ]; then
     # Use the new ABI on simulator, else we can't build
     export OBJCFLAGS="-fobjc-abi-version=2 -fobjc-legacy-dispatch ${OBJCFLAGS}"
 fi
 
-export LDFLAGS="-L${SDKROOT}/usr/lib -arch ${ARCH} -isysroot ${SDKROOT}"
+export LDFLAGS="-isysroot ${SDKROOT} -L${SDKROOT}/usr/lib -arch ${ACTUAL_ARCH}"
 
 if [ "$PLATFORM" = "OS" ]; then
-    EXTRA_CFLAGS="-arch ${ARCH}"
-    EXTRA_LDFLAGS="-arch ${ARCH}"
-if [ "$ARCH" != "arm64" ]; then
+    EXTRA_CFLAGS="-arch ${ACTUAL_ARCH}"
+    EXTRA_LDFLAGS="-arch ${ACTUAL_ARCH}"
+if [ "$ARCH" != "aarch64" ]; then
     EXTRA_CFLAGS+=" -mcpu=cortex-a8"
-    EXTRA_CFLAGS+=" -miphoneos-version-min=${SDK_MIN}"
-    EXTRA_LDFLAGS+=" -Wl,-ios_version_min,${SDK_MIN}"
-    export LDFLAGS="${LDFLAGS} -Wl,-ios_version_min,${SDK_MIN}"
+    EXTRA_CFLAGS+=" -${OSVERSIONMINCFLAG}=${SDK_MIN}"
+    EXTRA_LDFLAGS+=" -Wl,-${OSVERSIONMINLDFLAG},${SDK_MIN}"
+    export LDFLAGS="${LDFLAGS} -Wl,-${OSVERSIONMINLDFLAG},${SDK_MIN}"
 else
-    EXTRA_CFLAGS+=" -miphoneos-version-min=${SIXTYFOURBIT_SDK_MIN}"
-    EXTRA_LDFLAGS+=" -Wl,-ios_version_min,${SIXTYFOURBIT_SDK_MIN}"
-    export LDFLAGS="${LDFLAGS} -Wl,-ios_version_min,${SIXTYFOURBIT_SDK_MIN}"
+    EXTRA_CFLAGS+=" -${OSVERSIONMINCFLAG}=${SIXTYFOURBIT_SDK_MIN}"
+    EXTRA_LDFLAGS+=" -Wl,-${OSVERSIONMINLDFLAG},${SIXTYFOURBIT_SDK_MIN}"
+    export LDFLAGS="${LDFLAGS} -Wl,-${OSVERSIONMINLDFLAG},${SIXTYFOURBIT_SDK_MIN}"
 fi
 else
     EXTRA_CFLAGS="-arch ${ARCH}"
-    EXTRA_CFLAGS+=" -miphoneos-version-min=${SIXTYFOURBIT_SDK_MIN}"
-    EXTRA_LDFLAGS=" -Wl,-ios_version_min,${SIXTYFOURBIT_SDK_MIN}"
-    export LDFLAGS="${LDFLAGS} -Wl-ios_version_min,${SIXTYFOURBIT_SDK_MIN}"
+    EXTRA_CFLAGS+=" -${OSVERSIONMINCFLAG}=${SIXTYFOURBIT_SDK_MIN}"
+    EXTRA_LDFLAGS=" -Wl,-${OSVERSIONMINLDFLAG},${SIXTYFOURBIT_SDK_MIN}"
+    export LDFLAGS="${LDFLAGS} -v -Wl,-${OSVERSIONMINLDFLAG},${SIXTYFOURBIT_SDK_MIN}"
 fi
 
 
@@ -200,21 +233,27 @@ info "LD FLAGS SELECTED = '${LDFLAGS}'"
 spushd ${VLCROOT}/contrib
 
 echo ${VLCROOT}
-mkdir -p "${VLCROOT}/contrib/iPhone${PLATFORM}-${ARCH}"
-cd "${VLCROOT}/contrib/iPhone${PLATFORM}-${ARCH}"
+mkdir -p "${VLCROOT}/contrib/${OSSTYLE}${PLATFORM}-${ARCH}"
+cd "${VLCROOT}/contrib/${OSSTYLE}${PLATFORM}-${ARCH}"
 
 if [ "$PLATFORM" = "OS" ]; then
     export AS="gas-preprocessor.pl ${CC}"
     export ASCPP="gas-preprocessor.pl ${CC}"
     export CCAS="gas-preprocessor.pl ${CC}"
-    if [ "$ARCH" = "arm64" ]; then
+    if [ "$ARCH" = "aarch64" ]; then
         export GASPP_FIX_XCODE5=1
     fi
 else
     export ASCPP="xcrun as"
 fi
 
-../bootstrap --build=x86_64-apple-darwin11 --host=${TARGET} --prefix=${VLCROOT}/contrib/${TARGET}-${ARCH} --arch=${ARCH} --disable-gpl \
+if [ "$TVOS" = "yes" ]; then
+	TVOSOPTIONS="--disable-libarchive"
+else
+	TVOSOPTIONS=""
+fi
+
+../bootstrap --build=x86_64-apple-darwin11 --host=${TARGET} --prefix=${VLCROOT}/contrib/${OSSTYLE}-${TARGET}-${ARCH} --disable-gpl \
     --disable-disc --disable-sout \
     --disable-sdl \
     --disable-SDL_image \
@@ -243,14 +282,21 @@ fi
     --enable-ass \
     --disable-fontconfig \
     --disable-gpg-error \
+    --disable-vncclient \
+    --disable-gnutls \
     --disable-lua \
+    --disable-protobuf \
+    --disable-aribb24 \
+    --disable-aribb25 \
     --enable-vpx \
+    --enable-libdsm \
+    ${TVOSOPTIONS} \
     --enable-taglib > ${out}
 
 echo "EXTRA_CFLAGS += ${EXTRA_CFLAGS}" >> config.mak
 echo "EXTRA_LDFLAGS += ${EXTRA_LDFLAGS}" >> config.mak
 make fetch
-make
+make -j$MAKE_JOBS > ${out}
 spopd
 
 info "Bootstraping vlc"
@@ -282,23 +328,29 @@ else
 fi
 
 if [ "$SCARY" = "yes" ]; then
-	SCARYFLAG="--enable-dvbpsi --enable-avcodec"
+	SCARYFLAG="--enable-dvbpsi --enable-avcodec --disable-vpx"
 else
 	SCARYFLAG="--disable-dca --disable-dvbpsi --disable-avcodec --disable-avformat --disable-zvbi --enable-vpx"
 fi
 
+if [ "$TVOS" = "yes" ]; then
+	TVOSOPTIONS="--disable-neon"
+else
+	TVOSOPTIONS="--enable-neon"
+fi
+
 # Run configure only upon changes.
 if [ "${VLCROOT}/configure" -nt config.log -o \
      "${THIS_SCRIPT_PATH}" -nt config.log ]; then
 ${VLCROOT}/configure \
     --prefix="${PREFIX}" \
     --host="${TARGET}" \
-    --with-contrib="${VLCROOT}/contrib/${TARGET}-${ARCH}" \
+    --with-contrib="${VLCROOT}/contrib/${OSSTYLE}-${TARGET}-${ARCH}" \
     --enable-static \
     ${DEBUGFLAG} \
     ${SCARYFLAG} \
+    ${TVOSOPTIONS} \
     --disable-macosx \
-    --disable-macosx-dialog-provider \
     --disable-macosx-qtkit \
     --disable-macosx-eyetv \
     --disable-macosx-vlc-app \
@@ -318,9 +370,7 @@ ${VLCROOT}/configure \
     --disable-vlm \
     --disable-httpd \
     --disable-nls \
-    --disable-glx \
     --disable-sse \
-    --enable-neon \
     --disable-notify \
     --enable-live555 \
     --enable-realrtsp \
@@ -350,12 +400,9 @@ ${VLCROOT}/configure \
     --enable-taglib \
     --disable-mmx \
     --disable-addonmanagermodules \
-    --disable-mad > ${out} # MMX and SSE support requires llvm which is broken on Simulator
+    --disable-mad > ${out}
 fi
 
-CORE_COUNT=`sysctl -n machdep.cpu.core_count`
-let MAKE_JOBS=$CORE_COUNT+1
-
 info "Building libvlc"
 make -j$MAKE_JOBS > ${out}
 
@@ -364,7 +411,7 @@ make install > ${out}
 
 find ${PREFIX}/lib/vlc/plugins -name *.a -type f -exec cp '{}' ${PREFIX}/lib/vlc/plugins \;
 rm -rf "${PREFIX}/contribs"
-cp -R "${VLCROOT}/contrib/${TARGET}-${ARCH}" "${PREFIX}/contribs"
+cp -R "${VLCROOT}/contrib/${OSSTYLE}-${TARGET}-${ARCH}" "${PREFIX}/contribs"
 
 info "Removing unneeded modules"
 blacklist="
