@@ -2,7 +2,7 @@
  * vlc_extensions.h: Extensions (meta data, web information, ...)
  *****************************************************************************
  * Copyright (C) 2009-2010 VideoLAN and authors
- * $Id: 1b3b90a8b30afe0d11fb3b2c5e27f97619b13e81 $
+ * $Id: d49f629b5bba9c4bbe9541b4ef68519b4b0d2c76 $
  *
  * Authors: Jean-Philippe André < jpeg # videolan.org >
  *
@@ -311,8 +311,4 @@ struct extension_widget_t
     extension_dialog_t *p_dialog; ///< Parent dialog
 };
 
-VLC_API int dialog_ExtensionUpdate(vlc_object_t*, extension_dialog_t *);
-#define dialog_ExtensionUpdate(o, d) dialog_ExtensionUpdate(VLC_OBJECT(o), d)
-
 #endif /* VLC_EXTENSIONS_H */
-
