@@ -2,7 +2,7 @@
  * vlc_meta.h: Stream meta-data
  *****************************************************************************
  * Copyright (C) 2004 VLC authors and VideoLAN
- * $Id: 4292095290d804f6e22303de88ecae86be983fc0 $
+ * $Id: ad35faf3646509fead24b8542350677c2c8e18eb $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -56,14 +56,16 @@ typedef enum vlc_meta_type_t
     vlc_meta_Episode,
     vlc_meta_ShowName,
     vlc_meta_Actors,
+    vlc_meta_AlbumArtist,
+    vlc_meta_DiscNumber,
+    vlc_meta_DiscTotal
 } vlc_meta_type_t;
 
-#define VLC_META_TYPE_COUNT 24
+#define VLC_META_TYPE_COUNT 27
 
 #define ITEM_PREPARSED       1
-#define ITEM_ARTURL_FETCHED  2
-#define ITEM_ART_FETCHED     4
-#define ITEM_ART_NOTFOUND    8
+#define ITEM_ART_FETCHED     2
+#define ITEM_ART_NOTFOUND    4
 
 /**
  * Basic function to deal with meta
@@ -138,6 +140,8 @@ VLC_API int input_item_WriteMeta(vlc_object_t *, input_item_t *);
 #define vlc_meta_SetEpisode( meta, b )     vlc_meta_Set( meta, vlc_meta_Episode, b )
 #define vlc_meta_SetShowName( meta, b )    vlc_meta_Set( meta, vlc_meta_ShowName, b )
 #define vlc_meta_SetActors( meta, b )      vlc_meta_Set( meta, vlc_meta_Actors, b )
+#define vlc_meta_SetAlbumArtist( meta, b ) vlc_meta_Set( meta, vlc_meta_AlbumArtist, b )
+#define vlc_meta_SetDiscNumber( meta, b )  vlc_meta_Set( meta, vlc_meta_DiscNumber, b )
 
 #define VLC_META_TITLE              vlc_meta_TypeToLocalizedString( vlc_meta_Title )
 #define VLC_META_ARTIST             vlc_meta_TypeToLocalizedString( vlc_meta_Artist )
@@ -161,6 +165,8 @@ VLC_API int input_item_WriteMeta(vlc_object_t *, input_item_t *);
 #define VLC_META_EPISODE            vlc_meta_TypeToLocalizedString( vlc_meta_Episode )
 #define VLC_META_SHOW_NAME          vlc_meta_TypeToLocalizedString( vlc_meta_ShowName )
 #define VLC_META_ACTORS             vlc_meta_TypeToLocalizedString( vlc_meta_Actors )
+#define VLC_META_ALBUMARTIST        vlc_meta_TypeToLocalizedString( vlc_meta_AlbumArtist )
+#define VLC_META_DISCNUMBER         vlc_meta_TypeToLocalizedString( vlc_meta_DiscNumber )
 
 #define VLC_META_EXTRA_MB_ALBUMID   "MB_ALBUMID"
 
