@@ -2,7 +2,7 @@
  * vlc_update.h: VLC update download
  *****************************************************************************
  * Copyright © 2005-2007 VLC authors and VideoLAN
- * $Id: f7592212e7c5cc768e13ab835ad8132f92559fa4 $
+ * $Id: df7440546eed36181452b72d16dba5ae48bb4286 $
  *
  * Authors: Antoine Cellerier <dionoea -at- videolan -dot- org>
  *          Rafaël Carré <funman@videolanorg>
@@ -26,14 +26,12 @@
 #define VLC_UPDATE_H
 
 /**
- * \file
- * This file defines update API in vlc
- */
-
-/**
- * \defgroup update Update
- *
+ * \defgroup update Software updates
+ * \ingroup interface
+ * Over-the-air VLC software updates
  * @{
+ * \file
+ *VLC software update interface
  */
 
 #ifdef UPDATE_CHECK
