@@ -2,7 +2,7 @@
  * core.c: Core libvlc new API functions : initialization
  *****************************************************************************
  * Copyright (C) 2005 VLC authors and VideoLAN
- * $Id: fa5c4de6d0c0a623c9b271f8fd748da0e199c93c $
+ * $Id: c5ff9496919b14d3ba74eaa5845f35e2c1e1535f $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *
@@ -27,7 +27,7 @@
 
 #include "libvlc_internal.h"
 #include <vlc_modules.h>
-#include <vlc/libvlc.h>
+#include <vlc/vlc.h>
 
 #include <vlc_interface.h>
 #include <vlc_vlm.h>
