@@ -3,7 +3,7 @@
  * Also contains some internal utility functions
  *****************************************************************************
  * Copyright (C) 2005-2009 VLC authors and VideoLAN
- * $Id: 9301e320eaeff5d8ea6f206b1a06b9d9961599f3 $
+ * $Id: f063ddb5faaec4063e1cbca285c550a34b44b5ec $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *
@@ -43,7 +43,9 @@ struct libvlc_media_t
 
     vlc_cond_t parsed_cond;
     vlc_mutex_t parsed_lock;
+    vlc_mutex_t subitems_lock;
 
+    libvlc_media_parsed_status_t parsed_status;
     bool is_parsed;
     bool has_asked_preparse;
 };
