@@ -2,7 +2,7 @@
  * media_list.c: libvlc new API media list functions
  *****************************************************************************
  * Copyright (C) 2007 VLC authors and VideoLAN
- * $Id: 39bc354b0cd5fcc57f3c645d10c07f402f044f53 $
+ * $Id: 82fee60fdb24cdb9f4bc4ba063db6deeaa66e750 $
  *
  * Authors: Pierre d'Herbemont <pdherbemont # videolan.org>
  *
@@ -44,14 +44,6 @@ typedef enum EventPlaceInTime {
     EventDidHappen
 } EventPlaceInTime;
 
-//#define DEBUG_MEDIA_LIST
-
-#ifdef DEBUG_MEDIA_LIST
-# define trace( fmt, ... ) printf( "%s(): " fmt, __FUNCTION__, ##__VA_ARGS__ )
-#else
-# define trace( ... )
-#endif
-
 /*
  * Private functions
  */
@@ -74,7 +66,6 @@ notify_item_addition( libvlc_media_list_t * p_mlist,
     /* Construct the event */
     if( event_status == EventDidHappen )
     {
-        trace("item was added at index %d\n", index);
         event.type = libvlc_MediaListItemAdded;
         event.u.media_list_item_added.item = p_md;
         event.u.media_list_item_added.index = index;
@@ -106,7 +97,6 @@ notify_item_deletion( libvlc_media_list_t * p_mlist,
     /* Construct the event */
     if( event_status == EventDidHappen )
     {
-        trace("item at index %d was deleted\n", index);
         event.type = libvlc_MediaListItemDeleted;
         event.u.media_list_item_deleted.item = p_md;
         event.u.media_list_item_deleted.index = index;
@@ -122,6 +112,17 @@ notify_item_deletion( libvlc_media_list_t * p_mlist,
     libvlc_event_send( p_mlist->p_event_manager, &event );
 }
 
+/* LibVLC internal */
+void libvlc_media_list_internal_end_reached( libvlc_media_list_t * p_mlist )
+{
+    libvlc_event_t event;
+
+    event.type = libvlc_MediaListEndReached;
+
+    /* Send the event */
+    libvlc_event_send( p_mlist->p_event_manager, &event );
+}
+
 /**************************************************************************
  *       static mlist_is_writable (private)
  **************************************************************************/
@@ -159,7 +160,7 @@ libvlc_media_list_new( libvlc_instance_t * p_inst )
     }
 
     p_mlist->p_libvlc_instance = p_inst;
-    p_mlist->p_event_manager = libvlc_event_manager_new( p_mlist, p_inst );
+    p_mlist->p_event_manager = libvlc_event_manager_new( p_mlist );
     if( unlikely(p_mlist->p_event_manager == NULL) )
     {
         free(p_mlist);
@@ -168,15 +169,6 @@ libvlc_media_list_new( libvlc_instance_t * p_inst )
 
     p_mlist->b_read_only = false;
 
-    libvlc_event_manager_register_event_type( p_mlist->p_event_manager,
-            libvlc_MediaListItemAdded );
-    libvlc_event_manager_register_event_type( p_mlist->p_event_manager,
-            libvlc_MediaListWillAddItem );
-    libvlc_event_manager_register_event_type( p_mlist->p_event_manager,
-            libvlc_MediaListItemDeleted );
-    libvlc_event_manager_register_event_type( p_mlist->p_event_manager,
-            libvlc_MediaListWillDeleteItem );
-
     vlc_mutex_init( &p_mlist->object_lock );
     vlc_mutex_init( &p_mlist->refcount_lock ); // FIXME: spinlock?
 
@@ -184,7 +176,9 @@ libvlc_media_list_new( libvlc_instance_t * p_inst )
     assert( p_mlist->items.i_count == 0 );
     p_mlist->i_refcount = 1;
     p_mlist->p_md = NULL;
+    p_mlist->p_internal_md = NULL;
 
+    libvlc_retain( p_inst );
     return p_mlist;
 }
 
@@ -223,6 +217,7 @@ void libvlc_media_list_release( libvlc_media_list_t * p_mlist )
     vlc_mutex_destroy( &p_mlist->refcount_lock );
     vlc_array_clear( &p_mlist->items );
 
+    libvlc_release( p_mlist->p_libvlc_instance );
     free( p_mlist );
 }
 
@@ -249,8 +244,7 @@ libvlc_media_list_add_file_content( libvlc_media_list_t * p_mlist,
     input_item_t * p_input_item;
     libvlc_media_t * p_md;
 
-    p_input_item = input_item_NewExt( psz_uri,
-                                         _("Media Library"), 0, NULL, 0, -1 );
+    p_input_item = input_item_New( psz_uri, _("Media Library") );
 
     if( !p_input_item )
     {
@@ -285,6 +279,11 @@ void libvlc_media_list_set_media( libvlc_media_list_t * p_mlist,
 
 {
     vlc_mutex_lock( &p_mlist->object_lock );
+    if( p_mlist->p_internal_md || !mlist_is_writable(p_mlist) )
+    {
+        vlc_mutex_unlock( &p_mlist->object_lock );
+        return;
+    }
     libvlc_media_release( p_mlist->p_md );
     libvlc_media_retain( p_md );
     p_mlist->p_md = p_md;
@@ -306,7 +305,7 @@ libvlc_media_list_media( libvlc_media_list_t * p_mlist )
     libvlc_media_t *p_md;
 
     vlc_mutex_lock( &p_mlist->object_lock );
-    p_md = p_mlist->p_md;
+    p_md = p_mlist->p_internal_md ? p_mlist->p_internal_md : p_mlist->p_md;
     if( p_md )
         libvlc_media_retain( p_md );
     vlc_mutex_unlock( &p_mlist->object_lock );
@@ -334,13 +333,13 @@ int libvlc_media_list_add_media( libvlc_media_list_t * p_mlist,
 {
     if( !mlist_is_writable(p_mlist) )
         return -1;
-    _libvlc_media_list_add_media( p_mlist, p_md );
+    libvlc_media_list_internal_add_media( p_mlist, p_md );
     return 0;
 }
 
 /* LibVLC internal version */
-void _libvlc_media_list_add_media( libvlc_media_list_t * p_mlist,
-                                   libvlc_media_t * p_md )
+void libvlc_media_list_internal_add_media( libvlc_media_list_t * p_mlist,
+                                           libvlc_media_t * p_md )
 {
     libvlc_media_retain( p_md );
 
@@ -362,15 +361,14 @@ int libvlc_media_list_insert_media( libvlc_media_list_t * p_mlist,
 {
     if( !mlist_is_writable(p_mlist) )
         return -1;
-    _libvlc_media_list_insert_media( p_mlist, p_md, index );
+    libvlc_media_list_internal_insert_media( p_mlist, p_md, index );
     return 0;
 }
 
 /* LibVLC internal version */
-void _libvlc_media_list_insert_media(
-                                   libvlc_media_list_t * p_mlist,
-                                   libvlc_media_t * p_md,
-                                   int index )
+void libvlc_media_list_internal_insert_media( libvlc_media_list_t * p_mlist,
+                                              libvlc_media_t * p_md,
+                                              int index )
 {
     libvlc_media_retain( p_md );
 
@@ -389,12 +387,12 @@ int libvlc_media_list_remove_index( libvlc_media_list_t * p_mlist,
 {
     if( !mlist_is_writable(p_mlist) )
         return -1;
-    return _libvlc_media_list_remove_index( p_mlist, index );
+    return libvlc_media_list_internal_remove_index( p_mlist, index );
 }
 
 /* LibVLC internal version */
-int _libvlc_media_list_remove_index( libvlc_media_list_t * p_mlist,
-                                     int index )
+int libvlc_media_list_internal_remove_index( libvlc_media_list_t * p_mlist,
+                                             int index )
 {
     libvlc_media_t * p_md;
 
