@@ -3,7 +3,7 @@
  * Also contains some internal utility functions
  *****************************************************************************
  * Copyright (C) 2005-2009 VLC authors and VideoLAN
- * $Id: 185f183e9bd3ce70ea3fb6744e0eb79c4780d8da $
+ * $Id: f5f101b79a91a2ce9e03203b00124a6f44675911 $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *
@@ -30,10 +30,11 @@
 #endif
 
 #include <vlc/vlc.h>
-#include <vlc/libvlc_structures.h>
 #include <vlc/libvlc_media.h>
 #include <vlc_input.h>
 
+#include "../modules/audio_filter/equalizer_presets.h"
+
 struct libvlc_media_player_t
 {
     VLC_COMMON_MEMBERS
@@ -62,4 +63,13 @@ libvlc_track_description_t * libvlc_get_track_description(
         libvlc_media_player_t *p_mi,
         const char *psz_variable );
 
+/**
+ * Internal equalizer structure.
+ */
+struct libvlc_equalizer_t
+{
+    float f_preamp;
+    float f_amp[EQZ_BANDS_MAX];
+};
+
 #endif
