@@ -3,7 +3,7 @@
  *****************************************************************************
  * Copyright (C) 2005-2010 VLC authors and VideoLAN
  *
- * $Id: 0e3ccb5767938a40428393032cdbe672fcce89fe $
+ * $Id: b2c9b3435c17f015d509ec11049a8e874b1d12bf $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Filippo Carone <littlejohn@videolan.org>
@@ -230,16 +230,16 @@ unsigned libvlc_media_player_has_vout( libvlc_media_player_t *p_mi )
 
 float libvlc_video_get_scale( libvlc_media_player_t *mp )
 {
-    float f_scale = var_GetFloat (mp, "scale");
+    float f_scale = var_GetFloat (mp, "zoom");
     if (var_GetBool (mp, "autoscale"))
-        f_scale = 0.;
+        f_scale = 0.f;
     return f_scale;
 }
 
 void libvlc_video_set_scale( libvlc_media_player_t *p_mp, float f_scale )
 {
     if (isfinite(f_scale) && f_scale != 0.f)
-        var_SetFloat (p_mp, "scale", f_scale);
+        var_SetFloat (p_mp, "zoom", f_scale);
     var_SetBool (p_mp, "autoscale", f_scale == 0.f);
 
     /* Apply to current video outputs (if any) */
@@ -250,7 +250,7 @@ void libvlc_video_set_scale( libvlc_media_player_t *p_mp, float f_scale )
         vout_thread_t *p_vout = pp_vouts[i];
 
         if (isfinite(f_scale) && f_scale != 0.f)
-            var_SetFloat (p_vout, "scale", f_scale);
+            var_SetFloat (p_vout, "zoom", f_scale);
         var_SetBool (p_vout, "autoscale", f_scale == 0.f);
         vlc_object_release (p_vout);
     }
@@ -364,7 +364,7 @@ int64_t libvlc_video_get_spu_delay( libvlc_media_player_t *p_mi )
 
     if( p_input_thread )
     {
-        val = var_GetTime( p_input_thread, "spu-delay" );
+        val = var_GetInteger( p_input_thread, "spu-delay" );
         vlc_object_release( p_input_thread );
     }
     else
@@ -383,7 +383,7 @@ int libvlc_video_set_spu_delay( libvlc_media_player_t *p_mi,
 
     if( p_input_thread )
     {
-        var_SetTime( p_input_thread, "spu-delay", i_delay );
+        var_SetInteger( p_input_thread, "spu-delay", i_delay );
         vlc_object_release( p_input_thread );
         ret = 0;
     }
@@ -405,8 +405,8 @@ libvlc_track_description_t *
         libvlc_video_get_chapter_description( libvlc_media_player_t *p_mi,
                                               int i_title )
 {
-    char psz_title[12];
-    sprintf( psz_title,  "title %2i", i_title );
+    char psz_title[sizeof ("title ") + 3 * sizeof (int)];
+    sprintf( psz_title,  "title %2u", i_title );
     return libvlc_get_track_description( p_mi, psz_title );
 }
 
@@ -466,14 +466,11 @@ void libvlc_video_set_teletext( libvlc_media_player_t *p_mi, int i_page )
     }
 
     telx = var_GetInteger( p_input_thread, "teletext-es" );
-    if( telx >= 0 )
+    if( input_GetEsObjects( p_input_thread, telx, &p_zvbi, NULL, NULL )
+        == VLC_SUCCESS )
     {
-        if( input_GetEsObjects( p_input_thread, telx, &p_zvbi, NULL, NULL )
-            == VLC_SUCCESS )
-        {
-            var_SetInteger( p_zvbi, "vbi-page", i_page );
-            vlc_object_release( p_zvbi );
-        }
+        var_SetInteger( p_zvbi, "vbi-page", i_page );
+        vlc_object_release( p_zvbi );
     }
     vlc_object_release( p_input_thread );
 }
@@ -498,7 +495,7 @@ void libvlc_toggle_teletext( libvlc_media_player_t *p_mi )
     else
     {
         vlc_value_t list;
-        if( !var_Change( p_input_thread, "teletext-es", VLC_VAR_GETLIST, &list, NULL ) )
+        if( !var_Change( p_input_thread, "teletext-es", VLC_VAR_GETCHOICES, &list, NULL ) )
         {
             if( list.p_list->i_count > 0 )
                 var_SetInteger( p_input_thread, "spu-es", list.p_list->p_values[0].i_int );
@@ -555,6 +552,8 @@ int libvlc_video_set_track( libvlc_media_player_t *p_mi, int i_track )
     {
         if( i_track == val_list.p_list->p_values[i].i_int )
         {
+            if( var_SetBool( p_input_thread, "video", true ) < 0)
+                break;
             if( var_SetInteger( p_input_thread, "video-es", i_track ) < 0 )
                 break;
             i_ret = 0;
@@ -614,66 +613,63 @@ void libvlc_video_set_deinterlace( libvlc_media_player_t *p_mi,
 /* module helpers */
 /* ************** */
 
-
-static vlc_object_t *get_object( libvlc_media_player_t * p_mi,
-                                 const char *name )
+static bool find_sub_source_by_name( libvlc_media_player_t *p_mi, const char *restrict name )
 {
-    vlc_object_t *object;
     vout_thread_t *vout = GetVout( p_mi, 0 );
+    if (!vout)
+        return false;
 
-    if( vout )
+    char *psz_sources = var_GetString( vout, "sub-source" );
+    if( !psz_sources )
     {
-        object = vlc_object_find_name( vout, name );
-        vlc_object_release(vout);
-    }
-    else
-        object = NULL;
-
-    if( !object )
         libvlc_printerr( "%s not enabled", name );
-    return object;
-}
+        vlc_object_release( vout );
+        return false;
+    }
 
+    /* Find 'name'  */
+    char *p = strstr( psz_sources, name );
+    free( psz_sources );
+    vlc_object_release( vout );
+    return (p != NULL);
+}
 
 typedef const struct {
     const char name[20];
     unsigned type;
 } opt_t;
 
-
 static void
 set_int( libvlc_media_player_t *p_mi, const char *restrict name,
          const opt_t *restrict opt, int value )
 {
     if( !opt ) return;
 
-    if( !opt->type ) /* the enabler */
+    switch( opt->type )
     {
-        vout_thread_t *vout = GetVout( p_mi, 0 );
-        if (vout)
+        case 0: /* the enabler */
         {
-            vout_EnableFilter( vout, opt->name, value, false );
-            vlc_object_release( vout );
+            vout_thread_t *vout = GetVout( p_mi, 0 );
+            if (vout != NULL)
+            {   /* Fill sub-source */
+                vout_EnableFilter( vout, opt->name, value, false );
+                var_TriggerCallback( vout, "sub-source" );
+                vlc_object_release( vout );
+            }
+            break;
         }
-        return;
-    }
-
-    if( opt->type != VLC_VAR_INTEGER )
-    {
-        libvlc_printerr( "Invalid argument to %s in %s", name, "set int" );
-        return;
-    }
-
-    var_SetInteger(p_mi, opt->name, value);
-    vlc_object_t *object = get_object( p_mi, name );
-    if( object )
-    {
-        var_SetInteger(object, opt->name, value);
-        vlc_object_release( object );
+        case VLC_VAR_INTEGER:
+            var_SetInteger( p_mi, opt->name, value );
+            break;
+        case VLC_VAR_FLOAT:
+            var_SetFloat( p_mi, opt->name, value );
+            break;
+        default:
+            libvlc_printerr( "Invalid argument to %s in %s", name, "set int" );
+            return;
     }
 }
 
-
 static int
 get_int( libvlc_media_player_t *p_mi, const char *restrict name,
          const opt_t *restrict opt )
@@ -684,19 +680,19 @@ get_int( libvlc_media_player_t *p_mi, const char *restrict name,
     {
         case 0: /* the enabler */
         {
-            vlc_object_t *object = get_object( p_mi, name );
-            vlc_object_release( object );
-            return object != NULL;
+            bool b_enabled = find_sub_source_by_name( p_mi, name );
+            return b_enabled ? 1 : 0;
         }
     case VLC_VAR_INTEGER:
         return var_GetInteger(p_mi, opt->name);
+    case VLC_VAR_FLOAT:
+        return lroundf(var_GetFloat(p_mi, opt->name));
     default:
         libvlc_printerr( "Invalid argument to %s in %s", name, "get int" );
         return 0;
     }
 }
 
-
 static void
 set_float( libvlc_media_player_t *p_mi, const char *restrict name,
             const opt_t *restrict opt, float value )
@@ -710,23 +706,14 @@ set_float( libvlc_media_player_t *p_mi, const char *restrict name,
     }
 
     var_SetFloat( p_mi, opt->name, value );
-
-    vlc_object_t *object = get_object( p_mi, name );
-    if( object )
-    {
-        var_SetFloat(object, opt->name, value );
-        vlc_object_release( object );
-    }
 }
 
-
 static float
 get_float( libvlc_media_player_t *p_mi, const char *restrict name,
             const opt_t *restrict opt )
 {
     if( !opt ) return 0.0;
 
-
     if( opt->type != VLC_VAR_FLOAT )
     {
         libvlc_printerr( "Invalid argument to %s in %s", name, "get float" );
@@ -736,7 +723,6 @@ get_float( libvlc_media_player_t *p_mi, const char *restrict name,
     return var_GetFloat( p_mi, opt->name );
 }
 
-
 static void
 set_string( libvlc_media_player_t *p_mi, const char *restrict name,
             const opt_t *restrict opt, const char *restrict psz_value )
@@ -750,16 +736,8 @@ set_string( libvlc_media_player_t *p_mi, const char *restrict name,
     }
 
     var_SetString( p_mi, opt->name, psz_value );
-
-    vlc_object_t *object = get_object( p_mi, name );
-    if( object )
-    {
-        var_SetString(object, opt->name, psz_value );
-        vlc_object_release( object );
-    }
 }
 
-
 static char *
 get_string( libvlc_media_player_t *p_mi, const char *restrict name,
             const opt_t *restrict opt )
@@ -775,7 +753,6 @@ get_string( libvlc_media_player_t *p_mi, const char *restrict name,
     return var_GetString( p_mi, opt->name );
 }
 
-
 static const opt_t *
 marq_option_bynumber(unsigned option)
 {
@@ -800,8 +777,6 @@ marq_option_bynumber(unsigned option)
     return r;
 }
 
-static vlc_object_t *get_object( libvlc_media_player_t *, const char *);
-
 /*****************************************************************************
  * libvlc_video_get_marquee_int : get a marq option value
  *****************************************************************************/
@@ -841,7 +816,6 @@ void libvlc_video_set_marquee_string( libvlc_media_player_t *p_mi,
 
 /* logo module support */
 
-
 static const opt_t *
 logo_option_bynumber( unsigned option )
 {
@@ -865,11 +839,10 @@ logo_option_bynumber( unsigned option )
     return r;
 }
 
-
 void libvlc_video_set_logo_string( libvlc_media_player_t *p_mi,
                                    unsigned option, const char *psz_value )
 {
-    set_string( p_mi,"logo",logo_option_bynumber(option),psz_value );
+    set_string( p_mi,"logo",logo_option_bynumber(option), psz_value );
 }
 
 
@@ -895,12 +868,12 @@ adjust_option_bynumber( unsigned option )
 {
     static const opt_t optlist[] =
     {
-        { "adjust",               0 },
-        { "contrast",             VLC_VAR_FLOAT },
-        { "brightness",           VLC_VAR_FLOAT },
-        { "hue",                  VLC_VAR_INTEGER },
-        { "saturation",           VLC_VAR_FLOAT },
-        { "gamma",                VLC_VAR_FLOAT },
+        { "adjust",     0 },
+        { "contrast",   VLC_VAR_FLOAT },
+        { "brightness", VLC_VAR_FLOAT },
+        { "hue",        VLC_VAR_FLOAT },
+        { "saturation", VLC_VAR_FLOAT },
+        { "gamma",      VLC_VAR_FLOAT },
     };
     enum { num_opts = sizeof(optlist) / sizeof(*optlist) };
 
