@@ -2,7 +2,7 @@
  * vlm.c: libvlc new API VLM handling functions
  *****************************************************************************
  * Copyright (C) 2005 VLC authors and VideoLAN
- * $Id: aad066bc77f6ebaac8972e28005602bbff32aa6b $
+ * $Id: 8093dc8689607525c4610cea9c8a24d1f4d44e9b $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *
@@ -117,6 +117,7 @@ static void libvlc_vlm_release_internal( libvlc_instance_t *p_instance )
     p_instance->libvlc_vlm.p_event_manager = NULL;
     vlm_Delete( p_vlm );
     p_instance->libvlc_vlm.p_vlm = NULL;
+    libvlc_release( p_instance );
 }
 
 static int libvlc_vlm_init( libvlc_instance_t *p_instance )
@@ -124,42 +125,9 @@ static int libvlc_vlm_init( libvlc_instance_t *p_instance )
     if( !p_instance->libvlc_vlm.p_event_manager )
     {
         p_instance->libvlc_vlm.p_event_manager =
-            libvlc_event_manager_new( p_instance->libvlc_vlm.p_vlm, p_instance );
+            libvlc_event_manager_new( p_instance->libvlc_vlm.p_vlm );
         if( unlikely(p_instance->libvlc_vlm.p_event_manager == NULL) )
             return VLC_ENOMEM;
-        libvlc_event_manager_register_event_type(
-            p_instance->libvlc_vlm.p_event_manager,
-            libvlc_VlmMediaAdded );
-        libvlc_event_manager_register_event_type(
-            p_instance->libvlc_vlm.p_event_manager,
-            libvlc_VlmMediaRemoved );
-        libvlc_event_manager_register_event_type(
-            p_instance->libvlc_vlm.p_event_manager,
-            libvlc_VlmMediaChanged );
-        libvlc_event_manager_register_event_type(
-            p_instance->libvlc_vlm.p_event_manager,
-            libvlc_VlmMediaInstanceStarted );
-        libvlc_event_manager_register_event_type(
-            p_instance->libvlc_vlm.p_event_manager,
-            libvlc_VlmMediaInstanceStopped );
-        libvlc_event_manager_register_event_type(
-            p_instance->libvlc_vlm.p_event_manager,
-            libvlc_VlmMediaInstanceStatusInit );
-        libvlc_event_manager_register_event_type(
-            p_instance->libvlc_vlm.p_event_manager,
-            libvlc_VlmMediaInstanceStatusOpening );
-        libvlc_event_manager_register_event_type(
-            p_instance->libvlc_vlm.p_event_manager,
-            libvlc_VlmMediaInstanceStatusPlaying );
-        libvlc_event_manager_register_event_type(
-            p_instance->libvlc_vlm.p_event_manager,
-            libvlc_VlmMediaInstanceStatusPause );
-        libvlc_event_manager_register_event_type(
-            p_instance->libvlc_vlm.p_event_manager,
-            libvlc_VlmMediaInstanceStatusEnd );
-        libvlc_event_manager_register_event_type(
-            p_instance->libvlc_vlm.p_event_manager,
-            libvlc_VlmMediaInstanceStatusError );
     }
 
     if( !p_instance->libvlc_vlm.p_vlm )
@@ -174,6 +142,7 @@ static int libvlc_vlm_init( libvlc_instance_t *p_instance )
                          "intf-event", VlmEvent,
                          p_instance->libvlc_vlm.p_event_manager );
         p_instance->libvlc_vlm.pf_release = libvlc_vlm_release_internal;
+        libvlc_retain( p_instance );
     }
 
     return VLC_SUCCESS;
@@ -286,7 +255,7 @@ static char* recurse_answer( vlm_message_t *p_answer, const char* psz_delim,
                 strcmp( aw_child->psz_name, "inputs" ) == 0 ||
                 strcmp( aw_child->psz_name, "options" ) == 0 )
             {
-                char *psz_recurse = recurse_answer( aw_child, psz_childdelim, 1 ),
+                char *psz_recurse = recurse_answer( aw_child, psz_childdelim, 1 );
                 i_success = asprintf( &psz_tmp, "%s[%s%s%s]%c%s",
                                       psz_response, psz_childdelim, psz_recurse,
                                       psz_delim, c_comma, psz_delim );
@@ -300,7 +269,7 @@ static char* recurse_answer( vlm_message_t *p_answer, const char* psz_delim,
              */
             else
             {
-                char *psz_recurse = recurse_answer( aw_child, psz_childdelim, 0 ),
+                char *psz_recurse = recurse_answer( aw_child, psz_childdelim, 0 );
                 i_success = asprintf( &psz_tmp, "%s{%s%s%s%s}%c%s",
                                       psz_response, psz_childdelim, psz_nametag,
                                       psz_recurse, psz_delim, c_comma, psz_delim );
@@ -768,7 +737,7 @@ int libvlc_vlm_get_media_instance_seekable( libvlc_instance_t *p_instance,
 libvlc_event_manager_t *
 libvlc_vlm_get_event_manager( libvlc_instance_t *p_instance )
 {
-    vlm_t *p_vlm;
-    VLM_RET( p_vlm, NULL);
+    if( libvlc_vlm_init( p_instance ) )
+        return NULL;
     return p_instance->libvlc_vlm.p_event_manager;
 }
