@@ -2,7 +2,7 @@
  * bd.c: BluRay Disc support (uncrypted)
  *****************************************************************************
  * Copyright (C) 2009 VLC authors and VideoLAN
- * $Id: a73c88c39986ee3811ed8480fbefa8e076dcbda7 $
+ * $Id: 76e4998c8ac8bcff0e0ef016bb87a440ee2507c4 $
  *
  * Authors: Laurent Aimar <fenrir _AT_ videolan _DOT_ org>
  *
@@ -161,7 +161,10 @@ static int Open( vlc_object_t *p_this )
     /* Fill p_demux field */
     p_demux->p_sys = p_sys = malloc( sizeof(*p_sys) );
     if( !p_sys )
+    {
+        free( psz_base );
         return VLC_EGENERIC;
+    }
     p_sys->psz_base = psz_base;
     p_sys->b_shortname = b_shortname;
     TAB_INIT( p_sys->i_mpls, p_sys->pp_mpls );
@@ -610,6 +613,7 @@ static int SetPlayItem( demux_t *p_demux, int i_mpls, int i_play_item )
         if( p_sys->pp_clpi[i_clpi]->i_id == p_mpls_clpi->i_id )
             p_clpi = p_sys->pp_clpi[i_clpi];
     }
+    assert(p_clpi);
 
     const bool b_same_clpi = b_same_mpls && p_sys->p_clpi->i_id == p_clpi->i_id;
     stream_t *p_m2ts = NULL;
@@ -1043,7 +1047,7 @@ static void LoadMpls( demux_t *p_demux, const char *psz_name, int i_id )
 
     block_t *p_block = LoadBlock( p_demux, psz_name );
     if( !p_block )
-        goto error;
+        return;
 
     /* */
     bd_mpls_t *p_mpls = malloc( sizeof(*p_mpls) );
@@ -1111,8 +1115,8 @@ static void LoadMpls( demux_t *p_demux, const char *psz_name, int i_id )
 
 error:
     msg_Err( p_demux, "Failed loading %s", psz_name );
-    if( p_block )
-        block_Release( p_block );
+    free( p_mpls );
+    block_Release( p_block );
 }
 
 /* */
@@ -1137,7 +1141,7 @@ static void LoadClpi( demux_t *p_demux, const char *psz_name, int i_id )
 
     block_t *p_block = LoadBlock( p_demux, psz_name );
     if( !p_block )
-        goto error;
+        return;
 
     /* */
     bd_clpi_t *p_clpi = malloc( sizeof(*p_clpi) );
@@ -1185,8 +1189,8 @@ static void LoadClpi( demux_t *p_demux, const char *psz_name, int i_id )
 
 error:
     msg_Err( p_demux, "Failed loading %s", psz_name );
-    if( p_block )
-        block_Release( p_block );
+    free( p_clpi );
+    block_Release( p_block );
 }
 
 /* */
@@ -1255,7 +1259,7 @@ static es_out_id_t *EsOutAdd( es_out_t *p_out, const es_format_t *p_fmt )
     es_format_t fmt;
 
     es_format_Copy( &fmt, p_fmt );
-    fmt.i_priority = -2;
+    fmt.i_priority = ES_PRIORITY_NOT_SELECTABLE;
 
     for( int i = 0; i < p_item->i_stream; i++ )
     {
@@ -1267,7 +1271,7 @@ static es_out_id_t *EsOutAdd( es_out_t *p_out, const es_format_t *p_fmt )
         /* TODO improved priority for higher quality stream ?
          * if so, extending stream attributes parsing might be a good idea
          */
-        fmt.i_priority = 0;
+        fmt.i_priority = ES_PRIORITY_SELECTABLE_MIN;
 
 #if 0
         /* Useless, and beside not sure it is the right thing to do */
@@ -1297,7 +1301,7 @@ static es_out_id_t *EsOutAdd( es_out_t *p_out, const es_format_t *p_fmt )
         }
         break;
     }
-    if( fmt.i_priority < 0 )
+    if( fmt.i_priority < ES_PRIORITY_SELECTABLE_MIN )
         msg_Dbg( p_demux, "Hiding one stream (pid=%d)", fmt.i_id );
 
     /* */
