@@ -1,24 +1,24 @@
 /*****************************************************************************
- * filter.c : DirectShow access module for vlc
+ * filter.cpp : DirectShow access module for vlc
  *****************************************************************************
- * Copyright (C) 2002-2010 the VideoLAN team
- * $Id: eb5c7404c60027ae27f1d813e0dcde41856c54b9 $
+ * Copyright (C) 2002-2010 VLC authors and VideoLAN
+ * $Id: a6b099560c5af635663be8360f4aee1caadd4f3d $
  *
  * Author: Gildas Bazin <gbazin@videolan.org>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -42,6 +42,9 @@
 #include "vlc_dshow.h"
 
 #include <initguid.h>
+
+#include <new>
+
 DEFINE_GUID(MEDIASUBTYPE_HDYC ,0x43594448 /* CYDH */ , 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71);
 DEFINE_GUID(MEDIASUBTYPE_DIVX ,0x58564944 /* XVID */ , 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71);
 
@@ -236,7 +239,7 @@ CapturePin::~CapturePin()
  * @return S_OK if a sample was available, S_FALSE if no sample was
  * available
  */
-HRESULT CapturePin::CustomGetSamples( deque<VLCMediaSample> &external_queue )
+HRESULT CapturePin::CustomGetSamples( std::deque<VLCMediaSample> &external_queue )
 {
 #if 0 //def DEBUG_DSHOW
     msg_Dbg( p_input, "CapturePin::CustomGetSamples: %d samples in the queue", samples_queue.size());
@@ -561,7 +564,7 @@ STDMETHODIMP CapturePin::EnumMediaTypes( IEnumMediaTypes **ppEnum )
     msg_Dbg( p_input, "CapturePin::EnumMediaTypes" );
 #endif
 
-    *ppEnum = new CaptureEnumMediaTypes( p_input, this, NULL );
+    *ppEnum = new (std::nothrow) CaptureEnumMediaTypes( p_input, this, NULL );
 
     if( *ppEnum == NULL ) return E_OUTOFMEMORY;
 
@@ -849,7 +852,7 @@ STDMETHODIMP CaptureFilter::EnumPins( IEnumPins ** ppEnum )
 #endif
 
     /* Create a new ref counted enumerator */
-    *ppEnum = new CaptureEnumPins( p_input, this, NULL );
+    *ppEnum = new (std::nothrow) CaptureEnumPins( p_input, this, NULL );
     return *ppEnum == NULL ? E_OUTOFMEMORY : NOERROR;
 };
 STDMETHODIMP CaptureFilter::FindPin( LPCWSTR, IPin ** )
@@ -1021,7 +1024,7 @@ STDMETHODIMP CaptureEnumPins::Clone( IEnumPins **ppEnum )
     msg_Dbg( p_input, "CaptureEnumPins::Clone" );
 #endif
 
-    *ppEnum = new CaptureEnumPins( p_input, p_filter, this );
+    *ppEnum = new (std::nothrow) CaptureEnumPins( p_input, p_filter, this );
     if( *ppEnum == NULL ) return E_OUTOFMEMORY;
 
     return NOERROR;
@@ -1181,7 +1184,7 @@ STDMETHODIMP CaptureEnumMediaTypes::Clone( IEnumMediaTypes **ppEnum )
     msg_Dbg( p_input, "CaptureEnumMediaTypes::Clone" );
 #endif
 
-    *ppEnum = new CaptureEnumMediaTypes( p_input, p_pin, this );
+    *ppEnum = new (std::nothrow) CaptureEnumMediaTypes( p_input, p_pin, this );
     if( *ppEnum == NULL ) return E_OUTOFMEMORY;
 
     return NOERROR;
