@@ -3,7 +3,7 @@
  * CapturePin, CaptureFilter, CaptureEnumPins implementations
  *****************************************************************************
  * Copyright (C) 2002-2004, 2008 VLC authors and VideoLAN
- * $Id: 66402b57b1f5d792b538d1ac67b8902a82772ff4 $
+ * $Id: 69c3c0cf5e4c218074983813f95ce3c6ac5945af $
  *
  * Author: Gildas Bazin <gbazin@videolan.org>
  *
@@ -27,7 +27,6 @@
  *****************************************************************************/
 
 #include <deque>
-using namespace std;
 
 typedef struct VLCMediaSample
 {
@@ -62,7 +61,7 @@ class CapturePin: public IPin, public IMemInputPin
 
     AM_MEDIA_TYPE cx_media_type;
 
-    deque<VLCMediaSample> samples_queue;
+    std::deque<VLCMediaSample> samples_queue;
 
     long i_ref;
 
@@ -108,7 +107,7 @@ class CapturePin: public IPin, public IMemInputPin
 
     /* Custom methods */
     HRESULT CustomGetSample( VLCMediaSample * );
-    HRESULT CustomGetSamples( deque<VLCMediaSample> &external_queue );
+    HRESULT CustomGetSamples( std::deque<VLCMediaSample> &external_queue );
 
     AM_MEDIA_TYPE &CustomGetMediaType();
 };
