@@ -2,23 +2,23 @@
  * dv.c: Digital video/Firewire input (file: access plug-in)
  *****************************************************************************
  * Copyright (C) 2005 M2X
- * $Id: f9b7bf2a3359658f88038ac61ad132a7a3127c98 $
+ * $Id: 79717f96baa3521cb19b1f3e08e0539f0a05b4b6 $
  *
  * Authors: Jean-Paul Saman <jpsaman at m2x dot nl>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -28,18 +28,15 @@
 # include "config.h"
 #endif
 
+#include <assert.h>
+
 #include <vlc_common.h>
 #include <vlc_plugin.h>
 #include <vlc_access.h>
 
 #include <errno.h>
 #include <sys/types.h>
-#ifdef HAVE_UNISTD_H
-#   include <unistd.h>
-#elif defined( WIN32 ) && !defined( UNDER_CE )
-#   include <io.h>
-#endif
-
+#include <unistd.h>
 #include <sys/poll.h>
 
 #include <libraw1394/raw1394.h>
@@ -57,12 +54,12 @@ static block_t *Block( access_t * );
 static int Control( access_t *, int, va_list );
 
 vlc_module_begin ()
-    set_description( N_("Digital Video (Firewire/ieee1394)  input") )
+    set_description( N_("Digital Video (Firewire/ieee1394) input") )
     set_shortname( N_("DV") )
     set_category( CAT_INPUT )
     set_subcategory( SUBCAT_INPUT_ACCESS )
     set_capability( "access", 0 )
-    add_shortcut( "dv", "dv1394", "raw1394" )
+    add_shortcut( "dv", "raw1394" )
     set_callbacks( Open, Close )
 vlc_module_end ()
 
@@ -294,15 +291,7 @@ static int Control( access_t *p_access, int i_query, va_list args )
             AVCPause( p_access, p_access->p_sys->i_node );
             break;
 
-        case ACCESS_GET_TITLE_INFO:
-        case ACCESS_SET_TITLE:
-        case ACCESS_SET_SEEKPOINT:
-        case ACCESS_SET_PRIVATE_ID_STATE:
-        case ACCESS_GET_CONTENT_TYPE:
-            return VLC_EGENERIC;
-
         default:
-            msg_Warn( p_access, "unimplemented query in control" );
             return VLC_EGENERIC;
 
     }
@@ -316,7 +305,7 @@ static block_t *Block( access_t *p_access )
 
     vlc_mutex_lock( &p_sys->lock );
     p_block = p_sys->p_frame;
-    //msg_Dbg( p_access, "sending frame %p",p_block );
+    //msg_Dbg( p_access, "sending frame %p", (void *)p_block );
     p_sys->p_frame = NULL;
     vlc_mutex_unlock( &p_sys->lock );
 
@@ -347,7 +336,7 @@ static void* Raw1394EventThread( void *obj )
         while( ( result = poll( &p_sys->raw1394_poll, 1, -1 ) ) < 0 )
         {
             if( errno != EINTR )
-                msg_Err( p_access, "poll error: %m" );
+                msg_Err( p_access, "poll error: %s", vlc_strerror_c(errno) );
         }
 
         if( result > 0 && ( ( p_sys->raw1394_poll.revents & POLLIN )
@@ -359,8 +348,8 @@ static void* Raw1394EventThread( void *obj )
         }
     }
 
-    vlc_cleanup_run();
-    return NULL;
+    vlc_cleanup_pop();
+    vlc_assert_unreachable();
 }
 
 static enum raw1394_iso_disposition
@@ -402,7 +391,7 @@ Raw1394Handler(raw1394handle_t handle, unsigned char *data,
                 block_ChainAppend( &p_sys->p_frame, p_sys->p_ev->p_frame );
             }
             /* reset list */
-            p_sys->p_ev->p_frame = block_New( p_access, 144000 );
+            p_sys->p_ev->p_frame = block_Alloc( 144000 );
             p_sys->p_ev->pp_last = &p_sys->p_frame;
             vlc_mutex_unlock( &p_sys->lock );
         }
@@ -459,13 +448,15 @@ static int Raw1394GetNumPorts( access_t *p_access )
     /* get a raw1394 handle */
     if( !( handle = raw1394_new_handle() ) )
     {
-        msg_Err( p_access, "raw1394 - failed to get handle: %m." );
+        msg_Err( p_access, "raw1394 - failed to get handle: %s",
+                 vlc_strerror_c(errno) );
         return VLC_EGENERIC;
     }
 
     if( ( n_ports = raw1394_get_port_info( handle, pinf, 16 ) ) < 0 )
     {
-        msg_Err( p_access, "raw1394 - failed to get port info: %m." );
+        msg_Err( p_access, "raw1394 - failed to get port info: %s",
+                 vlc_strerror_c(errno) );
         raw1394_destroy_handle( handle );
         return VLC_EGENERIC;
     }
@@ -484,13 +475,15 @@ static raw1394handle_t Raw1394Open( access_t *p_access, int port )
     handle = raw1394_new_handle();
     if( !handle )
     {
-        msg_Err( p_access, "raw1394 - failed to get handle: %m." );
+        msg_Err( p_access, "raw1394 - failed to get handle: %s",
+                 vlc_strerror_c(errno) );
         return NULL;
     }
 
     if( ( n_ports = raw1394_get_port_info( handle, pinf, 16 ) ) < 0 )
     {
-        msg_Err( p_access, "raw1394 - failed to get port info: %m." );
+        msg_Err( p_access, "raw1394 - failed to get port info: %s",
+                 vlc_strerror_c(errno) );
         raw1394_destroy_handle( handle );
         return NULL;
     }
@@ -498,7 +491,8 @@ static raw1394handle_t Raw1394Open( access_t *p_access, int port )
     /* tell raw1394 which host adapter to use */
     if( raw1394_set_port( handle, port ) < 0 )
     {
-        msg_Err( p_access, "raw1394 - failed to set set port: %m." );
+        msg_Err( p_access, "raw1394 - failed to set set port: %s",
+                 vlc_strerror_c(errno) );
         return NULL;
     }
 
