@@ -3,7 +3,7 @@
  *****************************************************************************
  * Copyright (C) 2001-2006 VLC authors and VideoLAN
  * Copyright © 2006 Rémi Denis-Courmont
- * $Id: 54a2aa487814b1a1c1df23b71025f894bb84e917 $
+ * $Id: 9b8e9b59d0dab93c9ab90657f3be911b5ff1784f $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr> - original code
  *          Rémi Denis-Courmont <rem # videolan.org> - EPSV support
@@ -30,19 +30,22 @@
 # include "config.h"
 #endif
 
+#include <assert.h>
+#include <stdint.h>
+#include <errno.h>
+
 #include <vlc_common.h>
 #include <vlc_plugin.h>
-
-#include <assert.h>
-
 #include <vlc_access.h>
 #include <vlc_dialog.h>
-
+#include <vlc_input_item.h>
 #include <vlc_network.h>
 #include <vlc_url.h>
 #include <vlc_tls.h>
 #include <vlc_sout.h>
 #include <vlc_charset.h>
+#include <vlc_interrupt.h>
+#include <vlc_keystore.h>
 
 #ifndef IPPORT_FTP
 # define IPPORT_FTP 21u
@@ -62,16 +65,20 @@ static int  OutOpen ( vlc_object_t * );
 static void OutClose( vlc_object_t * );
 #endif
 
-#define USER_TEXT N_("FTP user name")
-#define USER_LONGTEXT N_("User name that will " \
-    "be used for the connection.")
-#define PASS_TEXT N_("FTP password")
-#define PASS_LONGTEXT N_("Password that will be " \
-    "used for the connection.")
+#define USER_TEXT N_("Username")
+#define USER_LONGTEXT N_("Username that will be used for the connection, " \
+        "if no username is set in the URL.")
+#define PASS_TEXT N_("Password")
+#define PASS_LONGTEXT N_("Password that will be used for the connection, " \
+        "if no username or password are set in URL.")
 #define ACCOUNT_TEXT N_("FTP account")
 #define ACCOUNT_LONGTEXT N_("Account that will be " \
     "used for the connection.")
 
+#define LOGIN_DIALOG_TITLE _("FTP authentication")
+#define LOGIN_DIALOG_TEXT _("Please enter a valid login and password for " \
+        "the ftp connexion to %s")
+
 vlc_module_begin ()
     set_shortname( "FTP" )
     set_description( N_("FTP input") )
@@ -105,11 +112,15 @@ vlc_module_end ()
 static ssize_t Read( access_t *, uint8_t *, size_t );
 static int Seek( access_t *, uint64_t );
 static int Control( access_t *, int, va_list );
+static int DirRead( access_t *, input_item_node_t * );
+static int DirControl( access_t *, int, va_list );
 #ifdef ENABLE_SOUT
 static int OutSeek( sout_access_out_t *, off_t );
 static ssize_t Write( sout_access_out_t *, block_t * );
 #endif
 
+static int LoginUserPwd( vlc_object_t *, access_sys_t *,
+                         const char *, const char *, bool * );
 static void FeaturesCheck( void *, const char * );
 
 typedef struct ftp_features_t
@@ -135,13 +146,12 @@ struct access_sys_t
     struct
     {
         vlc_tls_t   *p_tls;
-        v_socket_t  *p_vs;
         int          fd;
     } cmd, data;
 
     char       sz_epsv_ip[NI_MAXNUMERICHOST];
     bool       out;
-    bool       directory;
+    uint64_t   offset;
     uint64_t   size;
 };
 #define GET_OUT_SYS( p_this ) \
@@ -167,7 +177,9 @@ static int ftp_SendCommand( vlc_object_t *obj, access_sys_t *sys,
         return -1;
 
     msg_Dbg( obj, "sending request: \"%.*s\" (%d bytes)", val - 2, cmd, val );
-    if( net_Write( obj, sys->cmd.fd, sys->cmd.p_vs, cmd, val ) != val )
+    if( ((sys->cmd.p_tls != NULL)
+            ? vlc_tls_Write( sys->cmd.p_tls, cmd, val )
+            : net_Write( obj, sys->cmd.fd, cmd, val )) != val )
     {
         msg_Err( obj, "request failure" );
         val = -1;
@@ -202,7 +214,11 @@ static int ftp_RecvAnswer( vlc_object_t *obj, access_sys_t *sys,
     if( strp != NULL )
         *strp = NULL;
 
-    char *resp = net_Gets( obj, sys->cmd.fd, sys->cmd.p_vs );
+    char *resp;
+    if( sys->cmd.p_tls != NULL )
+        resp = vlc_tls_GetLine( sys->cmd.p_tls );
+    else
+        resp = net_Gets( obj, sys->cmd.fd );
     if( resp == NULL )
     {
         msg_Err( obj, "response failure" );
@@ -225,7 +241,11 @@ static int ftp_RecvAnswer( vlc_object_t *obj, access_sys_t *sys,
         *end = ' ';
         do
         {
-            char *line = net_Gets( obj, sys->cmd.fd, sys->cmd.p_vs );
+            char *line;
+            if( sys->cmd.p_tls != NULL )
+                line = vlc_tls_GetLine( sys->cmd.p_tls );
+            else
+                line = net_Gets( obj, sys->cmd.fd );
             if( line == NULL )
             {
                 msg_Err( obj, "response failure" );
@@ -263,10 +283,11 @@ static int ftp_RecvCommand( vlc_object_t *obj, access_sys_t *sys,
     return ftp_RecvAnswer( obj, sys, codep, strp, DummyLine, NULL );
 }
 
-static int ftp_StartStream( vlc_object_t *, access_sys_t *, uint64_t );
+static int ftp_StartStream( vlc_object_t *, access_sys_t *, uint64_t, bool );
 static int ftp_StopStream ( vlc_object_t *, access_sys_t * );
 
-static void readTLSMode( access_sys_t *p_sys, const char * psz_access )
+static int readTLSMode( vlc_object_t *obj, access_sys_t *p_sys,
+                        const char * psz_access )
 {
     if ( !strncmp( psz_access, "ftps", 4 ) )
         p_sys->tlsmode = IMPLICIT;
@@ -274,25 +295,29 @@ static void readTLSMode( access_sys_t *p_sys, const char * psz_access )
     if ( !strncmp( psz_access, "ftpes", 5 ) )
         p_sys->tlsmode = EXPLICIT;
     else
+    {
+        p_sys->p_creds = NULL;
         p_sys->tlsmode = NONE;
+        return 0;
+    }
+
+    p_sys->p_creds = vlc_tls_ClientCreate( obj );
+    return (p_sys->p_creds != NULL) ? 0 : -1;
 }
 
 static int createCmdTLS( vlc_object_t *p_access, access_sys_t *p_sys, int fd,
                          const char *psz_session_name )
 {
-    p_sys->p_creds = vlc_tls_ClientCreate( p_access );
-    if( p_sys->p_creds == NULL ) return -1;
-
     /* TLS/SSL handshake */
-    p_sys->cmd.p_tls = vlc_tls_ClientSessionCreate( p_sys->p_creds, fd,
-                                                    p_sys->url.psz_host,
-                                                    psz_session_name );
+    p_sys->cmd.p_tls = vlc_tls_ClientSessionCreateFD( p_sys->p_creds, fd,
+                                                      p_sys->url.psz_host,
+                                                      psz_session_name,
+                                                      NULL, NULL );
     if( p_sys->cmd.p_tls == NULL )
     {
         msg_Err( p_access, "cannot establish FTP/TLS session on command channel" );
         return -1;
     }
-    p_sys->cmd.p_vs = &p_sys->cmd.p_tls->sock;
 
     return 0;
 }
@@ -300,10 +325,7 @@ static int createCmdTLS( vlc_object_t *p_access, access_sys_t *p_sys, int fd,
 static void clearCmdTLS( access_sys_t *p_sys )
 {
     if ( p_sys->cmd.p_tls ) vlc_tls_SessionDelete( p_sys->cmd.p_tls );
-    if ( p_sys->p_creds ) vlc_tls_Delete( p_sys->p_creds );
     p_sys->cmd.p_tls = NULL;
-    p_sys->cmd.p_vs = NULL;
-    p_sys->p_creds = NULL;
 }
 
 static int Login( vlc_object_t *p_access, access_sys_t *p_sys )
@@ -316,8 +338,8 @@ static int Login( vlc_object_t *p_access, access_sys_t *p_sys )
     if( fd == -1 )
     {
         msg_Err( p_access, "connection failed" );
-        dialog_Fatal( p_access, _("Network interaction failed"), "%s",
-                        _("VLC could not connect with the given server.") );
+        vlc_dialog_display_error( p_access, _("Network interaction failed"), "%s",
+            _("VLC could not connect with the given server.") );
         return -1;
     }
 
@@ -332,8 +354,8 @@ static int Login( vlc_object_t *p_access, access_sys_t *p_sys )
     if( i_answer / 100 != 2 )
     {
         msg_Err( p_access, "connection rejected" );
-        dialog_Fatal( p_access, _("Network interaction failed"), "%s",
-                        _("VLC's connection to the given server was rejected.") );
+        vlc_dialog_display_error( p_access, _("Network interaction failed"), "%s",
+            _("VLC's connection to the given server was rejected.") );
         return -1;
     }
 
@@ -391,23 +413,42 @@ static int Login( vlc_object_t *p_access, access_sys_t *p_sys )
         }
     }
 
-    /* Send credentials over channel */
-    char *psz;
+    vlc_url_t url;
+    vlc_credential credential;
+    vlc_UrlParse( &url, ((access_t *)p_access)->psz_url );
+    vlc_credential_init( &credential, &url );
+    bool b_logged = false;
 
-    if( p_sys->url.psz_username && *p_sys->url.psz_username )
-        psz = strdup( p_sys->url.psz_username );
-    else
-        psz = var_InheritString( p_access, "ftp-user" );
-    if( !psz )
-        goto error;
-
-    if( ftp_SendCommand( p_access, p_sys, "USER %s", psz ) < 0 ||
-        ftp_RecvCommand( p_access, p_sys, &i_answer, NULL ) < 0 )
+    while( vlc_credential_get( &credential, p_access, "ftp-user", "ftp-pwd",
+                               LOGIN_DIALOG_TITLE, LOGIN_DIALOG_TEXT,
+                               url.psz_host )
+        && LoginUserPwd( p_access, p_sys, credential.psz_username,
+                         credential.psz_password, &b_logged ) == 0
+        && !b_logged );
+    if( b_logged )
     {
-        free( psz );
-        goto error;
+        vlc_credential_store( &credential, p_access );
+        vlc_credential_clean( &credential );
+        vlc_UrlClean( &url );
+        return 0;
     }
-    free( psz );
+    vlc_credential_clean( &credential );
+    vlc_UrlClean( &url );
+error:
+    clearCmdTLS( p_sys );
+    return -1;
+}
+
+static int LoginUserPwd( vlc_object_t *p_access, access_sys_t *p_sys,
+                         const char *psz_user, const char *psz_pwd,
+                         bool *p_logged )
+{
+    int i_answer;
+
+    /* Send credentials over channel */
+    if( ftp_SendCommand( p_access, p_sys, "USER %s", psz_user ) < 0 ||
+        ftp_RecvCommand( p_access, p_sys, &i_answer, NULL ) < 0 )
+        return -1;
 
     switch( i_answer / 100 )
     {
@@ -420,20 +461,10 @@ static int Login( vlc_object_t *p_access, access_sys_t *p_sys )
             break;
         case 3:
             msg_Dbg( p_access, "password needed" );
-            if( p_sys->url.psz_password && *p_sys->url.psz_password )
-                psz = strdup( p_sys->url.psz_password );
-            else
-                psz = var_InheritString( p_access, "ftp-pwd" );
-            if( !psz )
-                goto error;
 
-            if( ftp_SendCommand( p_access, p_sys, "PASS %s", psz ) < 0 ||
+            if( ftp_SendCommand( p_access, p_sys, "PASS %s", psz_pwd ) < 0 ||
                 ftp_RecvCommand( p_access, p_sys, &i_answer, NULL ) < 0 )
-            {
-                free( psz );
-                goto error;
-            }
-            free( psz );
+                return -1;
 
             switch( i_answer / 100 )
             {
@@ -441,6 +472,8 @@ static int Login( vlc_object_t *p_access, access_sys_t *p_sys )
                     msg_Dbg( p_access, "password accepted" );
                     break;
                 case 3:
+                {
+                    char *psz;
                     msg_Dbg( p_access, "account needed" );
                     psz = var_InheritString( p_access, "ftp-account" );
                     if( ftp_SendCommand( p_access, p_sys, "ACCT %s",
@@ -448,40 +481,36 @@ static int Login( vlc_object_t *p_access, access_sys_t *p_sys )
                         ftp_RecvCommand( p_access, p_sys, &i_answer, NULL ) < 0 )
                     {
                         free( psz );
-                        goto error;
+                        return -1;
                     }
                     free( psz );
 
                     if( i_answer / 100 != 2 )
                     {
                         msg_Err( p_access, "account rejected" );
-                        dialog_Fatal( p_access,
-                                      _("Network interaction failed"),
-                                      "%s", _("Your account was rejected.") );
-                        goto error;
+                        vlc_dialog_display_error( p_access,
+                          _("Network interaction failed"),
+                          "%s", _("Your account was rejected.") );
+                        return -1;
                     }
                     msg_Dbg( p_access, "account accepted" );
                     break;
+                }
 
                 default:
-                    msg_Err( p_access, "password rejected" );
-                    dialog_Fatal( p_access, _("Network interaction failed"),
-                                  "%s",  _("Your password was rejected.") );
-                    goto error;
+                    msg_Warn( p_access, "password rejected" );
+                    *p_logged = false;
+                    return 0;
             }
             break;
         default:
-            msg_Err( p_access, "user rejected" );
-            dialog_Fatal( p_access, _("Network interaction failed"), "%s",
-                        _("Your connection attempt to the server was rejected.") );
-            goto error;
+            msg_Warn( p_access, "user rejected" );
+            *p_logged = false;
+            return 0;
     }
 
+    *p_logged = true;
     return 0;
-
-error:
-    clearCmdTLS( p_sys );
-    return -1;
 }
 
 static void FeaturesCheck( void *opaque, const char *feature )
@@ -536,7 +565,8 @@ static int Connect( vlc_object_t *p_access, access_sys_t *p_sys )
             goto error;
     }
 
-    if( (p_sys->features.b_unicode ? IsUTF8 : IsASCII)(p_sys->url.psz_path) == NULL )
+    if( p_sys->url.psz_path &&
+        (p_sys->features.b_unicode ? IsUTF8 : IsASCII)(p_sys->url.psz_path) == NULL )
     {
         msg_Err( p_access, "unsupported path: \"%s\"", p_sys->url.psz_path );
         goto error;
@@ -568,7 +598,7 @@ static int parseURL( vlc_url_t *url, const char *path, enum tls_mode_e mode )
     while( *path == '/' )
         path++;
 
-    vlc_UrlParse( url, path, 0 );
+    vlc_UrlParse( url, path );
 
     if( url->psz_host == NULL || *url->psz_host == '\0' )
         return VLC_EGENERIC;
@@ -599,7 +629,7 @@ static int parseURL( vlc_url_t *url, const char *path, enum tls_mode_e mode )
         if( strchr( "iI", type[6] ) == NULL )
             return VLC_EGENERIC; /* ASCII and directory not supported */
     }
-    decode_URI( url->psz_path );
+    vlc_uri_decode( url->psz_path );
     return VLC_SUCCESS;
 }
 
@@ -612,14 +642,20 @@ static int InOpen( vlc_object_t *p_this )
     access_t     *p_access = (access_t*)p_this;
     access_sys_t *p_sys;
     char         *psz_arg;
+    bool          b_directory = false;
 
     /* Init p_access */
-    STANDARD_READ_ACCESS_INIT
+    access_InitFields( p_access );
+    p_sys = p_access->p_sys = (access_sys_t*)calloc( 1, sizeof( access_sys_t ) );
+    if( !p_sys )
+        return VLC_ENOMEM;
     p_sys->data.fd = -1;
     p_sys->out = false;
-    p_sys->directory = false;
-    p_sys->size = 0;
-    readTLSMode( p_sys, p_access->psz_access );
+    p_sys->offset = 0;
+    p_sys->size = UINT64_MAX;
+
+    if( readTLSMode( p_this, p_sys, p_access->psz_access ) )
+        goto exit_error;
 
     if( parseURL( &p_sys->url, p_access->psz_location, p_sys->tlsmode ) )
         goto exit_error;
@@ -629,7 +665,7 @@ static int InOpen( vlc_object_t *p_this )
 
     /* get size */
     if( p_sys->url.psz_path == NULL )
-        p_sys->directory = true;
+        b_directory = true;
     else
     if( ftp_SendCommand( p_this, p_sys, "SIZE %s", p_sys->url.psz_path ) < 0 )
         goto error;
@@ -650,10 +686,17 @@ static int InOpen( vlc_object_t *p_this )
         goto error;
     }
     else
-        p_sys->directory = true;
+        b_directory = true;
+
+    if( b_directory )
+    {
+        p_access->pf_readdir = DirRead;
+        p_access->pf_control = DirControl;
+    } else
+        ACCESS_SET_CALLBACKS( Read, NULL, Control, Seek ); \
 
     /* Start the 'stream' */
-    if( ftp_StartStream( p_this, p_sys, 0 ) < 0 )
+    if( ftp_StartStream( p_this, p_sys, 0, b_directory ) < 0 )
     {
         msg_Err( p_this, "cannot retrieve file" );
         clearCmdTLS( p_sys );
@@ -669,6 +712,7 @@ error:
 
 exit_error:
     vlc_UrlClean( &p_sys->url );
+    vlc_tls_Delete( p_sys->p_creds );
     free( p_sys );
     return VLC_EGENERIC;
 }
@@ -686,7 +730,9 @@ static int OutOpen( vlc_object_t *p_this )
     /* Init p_access */
     p_sys->data.fd = -1;
     p_sys->out = true;
-    readTLSMode( p_sys, p_access->psz_access );
+
+    if( readTLSMode( p_this, p_sys, p_access->psz_access ) )
+        goto exit_error;
 
     if( parseURL( &p_sys->url, p_access->psz_path, p_sys->tlsmode ) )
         goto exit_error;
@@ -700,7 +746,7 @@ static int OutOpen( vlc_object_t *p_this )
         goto exit_error;
 
     /* Start the 'stream' */
-    if( ftp_StartStream( p_this, p_sys, 0 ) < 0 )
+    if( ftp_StartStream( p_this, p_sys, 0, false ) < 0 )
     {
         msg_Err( p_access, "cannot store file" );
         clearCmdTLS( p_sys );
@@ -716,6 +762,7 @@ static int OutOpen( vlc_object_t *p_this )
 
 exit_error:
     vlc_UrlClean( &p_sys->url );
+    vlc_tls_Delete( p_sys->p_creds );
     free( p_sys );
     return VLC_EGENERIC;
 }
@@ -743,6 +790,7 @@ static void Close( vlc_object_t *p_access, access_sys_t *p_sys )
 
     /* free memory */
     vlc_UrlClean( &p_sys->url );
+    vlc_tls_Delete( p_sys->p_creds );
     free( p_sys );
 }
 
@@ -767,7 +815,7 @@ static int _Seek( vlc_object_t *p_access, access_sys_t *p_sys, uint64_t i_pos )
     msg_Dbg( p_access, "seeking to %"PRIu64, i_pos );
 
     ftp_StopStream( (vlc_object_t *)p_access, p_sys );
-    if( ftp_StartStream( (vlc_object_t *)p_access, p_sys, i_pos ) < 0 )
+    if( ftp_StartStream( (vlc_object_t *)p_access, p_sys, i_pos, false ) < 0 )
         return VLC_EGENERIC;
 
     return VLC_SUCCESS;
@@ -775,12 +823,14 @@ static int _Seek( vlc_object_t *p_access, access_sys_t *p_sys, uint64_t i_pos )
 
 static int Seek( access_t *p_access, uint64_t i_pos )
 {
-    int val = _Seek( (vlc_object_t *)p_access, p_access->p_sys, i_pos );
+    access_sys_t *p_sys = p_access->p_sys;
+
+    int val = _Seek( (vlc_object_t *)p_access, p_sys, i_pos );
     if( val )
         return val;
 
     p_access->info.b_eof = false;
-    p_access->info.i_pos = i_pos;
+    p_sys->offset = i_pos;
 
     return VLC_SUCCESS;
 }
@@ -798,6 +848,7 @@ static int OutSeek( sout_access_out_t *p_access, off_t i_pos )
 static ssize_t Read( access_t *p_access, uint8_t *p_buffer, size_t i_len )
 {
     access_sys_t *p_sys = p_access->p_sys;
+    int i_read;
 
     assert( p_sys->data.fd != -1 );
     assert( !p_sys->out );
@@ -805,36 +856,81 @@ static ssize_t Read( access_t *p_access, uint8_t *p_buffer, size_t i_len )
     if( p_access->info.b_eof )
         return 0;
 
-    if( p_sys->directory )
+    if( p_sys->data.p_tls != NULL )
+        i_read = vlc_tls_Read( p_sys->data.p_tls, p_buffer, i_len, false );
+    else
+        i_read = vlc_recv_i11e( p_sys->data.fd, p_buffer, i_len, 0 );
+
+    if( i_read > 0 )
+        p_sys->offset += i_read;
+    else if( i_read == 0 )
+        p_access->info.b_eof = true;
+    else if( errno != EINTR && errno != EAGAIN )
     {
-        char *psz_line = net_Gets( p_access, p_sys->data.fd, p_sys->data.p_vs );
-        if( !psz_line )
-        {
-            p_access->info.b_eof = true;
-            return 0;
-        }
+        msg_Err( p_access, "receive error: %s", vlc_strerror_c(errno) );
+        p_access->info.b_eof = true;
+    }
+
+    return i_read;
+}
+
+/*****************************************************************************
+ * DirRead:
+ *****************************************************************************/
+static int DirRead (access_t *p_access, input_item_node_t *p_current_node)
+{
+    access_sys_t *p_sys = p_access->p_sys;
+    int i_ret = VLC_SUCCESS;
+
+    assert( p_sys->data.fd != -1 );
+    assert( !p_sys->out );
+
+    struct access_fsdir fsdir;
+    access_fsdir_init( &fsdir, p_access, p_current_node );
+
+    while (i_ret == VLC_SUCCESS)
+    {
+        char *psz_line;
+        if( p_sys->data.p_tls != NULL )
+            psz_line = vlc_tls_GetLine( p_sys->data.p_tls );
         else
-        {
-            snprintf( (char*)p_buffer, i_len, "%s://%s:%d/%s/%s\n",
+            psz_line = net_Gets( p_access, p_sys->data.fd );
+
+        if( psz_line == NULL )
+            break;
+
+        char *psz_uri;
+        if( asprintf( &psz_uri, "%s://%s:%d%s%s/%s",
                       ( p_sys->tlsmode == NONE ) ? "ftp" :
                       ( ( p_sys->tlsmode == IMPLICIT ) ? "ftps" : "ftpes" ),
                       p_sys->url.psz_host, p_sys->url.i_port,
-                      p_sys->url.psz_path, psz_line );
-            free( psz_line );
-            return strlen( (const char *)p_buffer );
+                      p_sys->url.psz_path ? "/" : "",
+                      p_sys->url.psz_path ? p_sys->url.psz_path : "",
+                      psz_line ) != -1 )
+        {
+            i_ret = access_fsdir_additem( &fsdir, psz_uri, psz_line,
+                                          ITEM_TYPE_UNKNOWN, ITEM_NET );
+            free( psz_uri );
         }
+        free( psz_line );
     }
-    else
-    {
-        int i_read = net_Read( p_access, p_sys->data.fd, p_sys->data.p_vs,
-                               p_buffer, i_len, false );
-        if( i_read == 0 )
-            p_access->info.b_eof = true;
-        else if( i_read > 0 )
-            p_access->info.i_pos += i_read;
 
-        return i_read;
+    access_fsdir_finish( &fsdir, i_ret == VLC_SUCCESS );
+    return i_ret;
+}
+
+static int DirControl( access_t *p_access, int i_query, va_list args )
+{
+    switch( i_query )
+    {
+    case ACCESS_IS_DIRECTORY:
+        *va_arg( args, bool * ) = true; /* might loop */
+        break;
+    default:
+        return access_vaDirectoryControlHelper( p_access, i_query, args );
     }
+
+    return VLC_SUCCESS;
 }
 
 /*****************************************************************************
@@ -852,8 +948,12 @@ static ssize_t Write( sout_access_out_t *p_access, block_t *p_buffer )
     {
         block_t *p_next = p_buffer->p_next;;
 
-        i_write += net_Write( p_access, p_sys->data.fd, p_sys->data.p_vs,
-                              p_buffer->p_buffer, p_buffer->i_buffer );
+        if( p_sys->data.p_tls != NULL )
+            i_write += vlc_tls_Write( p_sys->data.p_tls,
+                                      p_buffer->p_buffer, p_buffer->i_buffer );
+        else
+            i_write += net_Write( p_access, p_sys->data.fd,
+                                  p_buffer->p_buffer, p_buffer->i_buffer );
         block_Release( p_buffer );
 
         p_buffer = p_next;
@@ -875,7 +975,7 @@ static int Control( access_t *p_access, int i_query, va_list args )
     {
         case ACCESS_CAN_SEEK:
             pb_bool = (bool*)va_arg( args, bool* );
-            *pb_bool = !p_access->p_sys->directory;
+            *pb_bool = true;
             break;
         case ACCESS_CAN_FASTSEEK:
             pb_bool = (bool*)va_arg( args, bool* );
@@ -890,6 +990,8 @@ static int Control( access_t *p_access, int i_query, va_list args )
             *pb_bool = true;    /* FIXME */
             break;
         case ACCESS_GET_SIZE:
+            if( p_access->p_sys->size == UINT64_MAX )
+                return VLC_EGENERIC;
             *va_arg( args, uint64_t * ) = p_access->p_sys->size;
             break;
 
@@ -902,7 +1004,7 @@ static int Control( access_t *p_access, int i_query, va_list args )
         case ACCESS_SET_PAUSE_STATE:
             pb_bool = (bool*)va_arg( args, bool* );
             if ( !pb_bool )
-              return Seek( p_access, p_access->info.i_pos );
+                 return Seek( p_access, p_access->p_sys->offset );
             break;
 
         default:
@@ -913,7 +1015,7 @@ static int Control( access_t *p_access, int i_query, va_list args )
 }
 
 static int ftp_StartStream( vlc_object_t *p_access, access_sys_t *p_sys,
-                            uint64_t i_start )
+                            uint64_t i_start, bool b_directory )
 {
     char psz_ipv4[16], *psz_ip = p_sys->sz_epsv_ip;
     int  i_answer;
@@ -939,7 +1041,10 @@ static int ftp_StartStream( vlc_object_t *p_access, access_sys_t *p_sys,
 
     if( *psz_ip )
     {
-        if( sscanf( psz_parser, "(%*3c%u", &i_port ) < 1 )
+        char psz_fmt[7] = "(|||%u";
+        psz_fmt[1] = psz_fmt[2] = psz_fmt[3] = psz_parser[1];
+
+        if( sscanf( psz_parser, psz_fmt, &i_port ) < 1 )
         {
             free( psz_arg );
             msg_Err( p_access, "cannot parse passive mode response" );
@@ -994,13 +1099,13 @@ static int ftp_StartStream( vlc_object_t *p_access, access_sys_t *p_sys,
     msg_Dbg( p_access, "connection with \"%s:%d\" successful",
              psz_ip, i_port );
 
-    if( p_sys->directory )
+    if( b_directory )
     {
         if( ftp_SendCommand( p_access, p_sys, "NLST" ) < 0 ||
             ftp_RecvCommand( p_access, p_sys, NULL, &psz_arg ) > 2 )
         {
             msg_Err( p_access, "cannot list directory contents" );
-        return VLC_EGENERIC;
+            return VLC_EGENERIC;
         }
     }
     else
@@ -1021,17 +1126,17 @@ static int ftp_StartStream( vlc_object_t *p_access, access_sys_t *p_sys,
     {
         /* FIXME: Do Reuse TLS Session */
         /* TLS/SSL handshake */
-        p_sys->data.p_tls = vlc_tls_ClientSessionCreate( p_sys->p_creds,
+        p_sys->data.p_tls = vlc_tls_ClientSessionCreateFD( p_sys->p_creds,
                             p_sys->data.fd, p_sys->url.psz_host,
                             ( p_sys->tlsmode == EXPLICIT ) ? "ftpes-data"
-                                                           : "ftps-data" );
+                                                           : "ftps-data",
+                                                         NULL, NULL );
         if( p_sys->data.p_tls == NULL )
         {
             msg_Err( p_access, "cannot establish FTP/TLS session for data" \
                              ": server not allowing new session ?" );
             return VLC_EGENERIC;
         }
-        p_sys->data.p_vs = &p_sys->data.p_tls->sock;
     }
     else
         shutdown( p_sys->data.fd, p_sys->out ? SHUT_RD : SHUT_WR );
@@ -1051,7 +1156,6 @@ static int ftp_StopStream ( vlc_object_t *p_access, access_sys_t *p_sys )
         }
         p_sys->data.fd = -1;
         p_sys->data.p_tls = NULL;
-        p_sys->data.p_vs = NULL;
         return VLC_EGENERIC;
     }
 
@@ -1061,7 +1165,6 @@ static int ftp_StopStream ( vlc_object_t *p_access, access_sys_t *p_sys )
         net_Close( p_sys->data.fd );
         p_sys->data.fd = -1;
         p_sys->data.p_tls = NULL;
-        p_sys->data.p_vs = NULL;
         /* Read the final response from RETR/STOR, i.e. 426 or 226 */
         ftp_RecvCommand( p_access, p_sys, NULL, NULL );
     }
