@@ -2,7 +2,7 @@
  * imem.c : Memory input for VLC
  *****************************************************************************
  * Copyright (C) 2009-2010 Laurent Aimar
- * $Id: 6c1bd735c0b51375ffde5d85d455c04c31c9b87f $
+ * $Id: b3f1cdaecc481643ce29dc5c687e0e6617ccd6e4 $
  *
  * Author: Laurent Aimar <fenrir _AT_ videolan _DOT org>
  *
@@ -275,7 +275,8 @@ static int OpenCommon(vlc_object_t *object, imem_sys_t **sys_ptr, const char *ps
     sys->source.cookie = var_InheritString(object, "imem-cookie");
 
     msg_Dbg(object, "Using get(%p), release(%p), data(%p), cookie(%s)",
-            sys->source.get, sys->source.release, sys->source.data,
+            (void *)sys->source.get, (void *)sys->source.release,
+            sys->source.data,
             sys->source.cookie ? sys->source.cookie : "(null)");
 
     /* */
@@ -309,7 +310,6 @@ static int OpenAccess(vlc_object_t *object)
     access->pf_block   = Block;
     access->pf_seek    = NULL;
     access->p_sys      = (access_sys_t*)sys;
-    access->info.i_size = var_InheritInteger(object, "imem-size");
 
     return VLC_SUCCESS;
 }
@@ -344,6 +344,11 @@ static int ControlAccess(access_t *access, int i_query, va_list args)
         *b = true;
         return VLC_SUCCESS;
     }
+    case ACCESS_GET_SIZE: {
+        uint64_t *s = va_arg(args, uint64_t *);
+        *s = var_InheritInteger(access, "imem-size");
+        return *s ? VLC_SUCCESS : VLC_EGENERIC;
+    }
     case ACCESS_GET_PTS_DELAY: {
         int64_t *delay = va_arg(args, int64_t *);
         *delay = DEFAULT_PTS_DELAY; /* FIXME? */
@@ -352,13 +357,6 @@ static int ControlAccess(access_t *access, int i_query, va_list args)
     case ACCESS_SET_PAUSE_STATE:
         return VLC_SUCCESS;
 
-    case ACCESS_GET_TITLE_INFO:
-    case ACCESS_SET_TITLE:
-    case ACCESS_SET_SEEKPOINT:
-    case ACCESS_SET_PRIVATE_ID_STATE:
-    case ACCESS_GET_META:
-    case ACCESS_GET_PRIVATE_ID_STATE:
-    case ACCESS_GET_CONTENT_TYPE:
     default:
         return VLC_EGENERIC;
     }
