@@ -32,6 +32,7 @@
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/eventfd.h>
 #include <fcntl.h>
 #include <errno.h>
 
@@ -60,7 +61,6 @@
 #define SDIAUDIO_SAMPLESIZE_FILE "/sys/class/sdiaudio/sdiaudiorx%u/sample_size"
 #define SDIAUDIO_CHANNELS_FILE  "/sys/class/sdiaudio/sdiaudiorx%u/channels"
 #define NB_VBUFFERS             2
-#define READ_TIMEOUT            80000
 #define CLOCK_GAP               INT64_C(500000)
 #define START_DATE              INT64_C(4294967296)
 
@@ -154,10 +154,13 @@ struct demux_sys_t
     int          i_id_video;
     es_out_id_t  *p_es_video;
     hdsdi_audio_t p_audios[MAX_AUDIOS];
+
+    pthread_t thread;
+    int evfd;
 };
 
 static int Control( demux_t *, int, va_list );
-static int Demux( demux_t * );
+static void *Demux( void * );
 
 static int InitCapture( demux_t *p_demux );
 static void CloseCapture( demux_t *p_demux );
@@ -173,8 +176,6 @@ static int Open( vlc_object_t *p_this )
     char        *psz_parser;
 
     /* Fill p_demux field */
-    p_demux->pf_demux = Demux;
-    p_demux->pf_control = Control;
     p_demux->p_sys = p_sys = calloc( 1, sizeof( demux_sys_t ) );
     if( unlikely(!p_sys) )
         return VLC_ENOMEM;
@@ -246,13 +247,22 @@ static int Open( vlc_object_t *p_this )
 
     p_sys->i_link = var_InheritInteger( p_demux, "linsys-hdsdi-link" );
 
-    if( InitCapture( p_demux ) != VLC_SUCCESS )
+    p_sys->evfd = eventfd( 0, EFD_CLOEXEC );
+    if( p_sys->evfd == -1 )
+        goto error;
+
+    if( pthread_create( &p_sys->thread, NULL, Demux, p_demux ) )
     {
-        free( p_sys );
-        return VLC_EGENERIC;
+        vlc_close( p_sys->evfd );
+        goto error;
     }
 
+    p_demux->pf_demux = NULL;
+    p_demux->pf_control = Control;
     return VLC_SUCCESS;
+error:
+    free( p_sys );
+    return VLC_EGENERIC;
 }
 
 /*****************************************************************************
@@ -263,16 +273,26 @@ static void Close( vlc_object_t *p_this )
     demux_t     *p_demux = (demux_t *)p_this;
     demux_sys_t *p_sys = p_demux->p_sys;
 
-    CloseCapture( p_demux );
+    write( p_sys->evfd, &(uint64_t){ 1 }, sizeof (uint64_t));
+    pthread_join( p_sys->thread, NULL );
+    vlc_close( p_sys->evfd );
     free( p_sys );
 }
 
 /*****************************************************************************
  * DemuxDemux:
  *****************************************************************************/
-static int Demux( demux_t *p_demux )
+static void *Demux( void *opaque )
 {
-    return ( Capture( p_demux ) == VLC_SUCCESS );
+    demux_t *p_demux = opaque;
+
+    if( InitCapture( p_demux ) != VLC_SUCCESS )
+        return NULL;
+
+    while( Capture( p_demux ) == VLC_SUCCESS );
+
+    CloseCapture( p_demux );
+    return NULL;
 }
 
 /*****************************************************************************
@@ -647,7 +667,7 @@ static ssize_t WriteULSysfs( const char *psz_fmt, unsigned int i_link,
         return i_fd;
 
     i_ret = write( i_fd, psz_data, strlen(psz_data) + 1 );
-    close( i_fd );
+    vlc_close( i_fd );
     return i_ret;
 }
 
@@ -661,7 +681,6 @@ static int InitCapture( demux_t *p_demux )
     char psz_vdev[MAXLEN];
 
     snprintf( psz_vdev, sizeof(psz_vdev), SDIVIDEO_DEVICE, p_sys->i_link );
-    psz_vdev[sizeof(psz_vdev) - 1] = '\0';
     if ( (p_sys->i_vfd = vlc_open( psz_vdev, O_RDONLY ) ) < 0 )
     {
         msg_Err( p_demux, "couldn't open device %s", psz_vdev );
@@ -669,19 +688,17 @@ static int InitCapture( demux_t *p_demux )
     }
 
     /* Wait for standard to settle down */
-    while ( vlc_object_alive(p_demux) )
+    struct pollfd pfd[2];
+
+    pfd[0].fd = p_sys->i_vfd;
+    pfd[0].events = POLLPRI;
+    pfd[1].fd = p_sys->evfd;
+    pfd[1].events = POLLIN;
+
+    for( ;; )
     {
-        struct pollfd pfd[1];
-
-        pfd[0].fd = p_sys->i_vfd;
-        pfd[0].events = POLLIN | POLLPRI;
-
-        if ( poll( pfd, 1, READ_TIMEOUT ) < 0 )
-        {
-            msg_Warn( p_demux, "couldn't poll(): %s", vlc_strerror_c(errno) );
-            close( p_sys->i_vfd );
-            return VLC_EGENERIC;
-        }
+        if( poll( pfd, 2, -1 ) < 0 )
+            continue;
 
         if ( pfd[0].revents & POLLPRI )
         {
@@ -707,11 +724,12 @@ static int InitCapture( demux_t *p_demux )
                 }
             }
         }
-    }
-    if ( !vlc_object_alive(p_demux) )
-    {
-        close( p_sys->i_vfd );
-        return VLC_EGENERIC;
+
+        if( pfd[1].revents )
+        {
+            vlc_close( p_sys->i_vfd );
+            return VLC_EGENERIC;
+        }
     }
 
     if ( ioctl( p_sys->i_vfd, SDIVIDEO_IOC_RXGETVIDSTATUS, &p_sys->i_standard )
@@ -719,10 +737,10 @@ static int InitCapture( demux_t *p_demux )
     {
         msg_Warn( p_demux, "couldn't SDIVIDEO_IOC_RXGETVIDSTATUS: %s",
                   vlc_strerror_c(errno) );
-        close( p_sys->i_vfd );
+        vlc_close( p_sys->i_vfd );
         return VLC_EGENERIC;
     }
-    close( p_sys->i_vfd );
+    vlc_close( p_sys->i_vfd );
 
     if ( InitVideo( p_demux ) != VLC_SUCCESS )
         return VLC_EGENERIC;
@@ -735,7 +753,6 @@ static int InitCapture( demux_t *p_demux )
         char psz_adev[MAXLEN];
 
         snprintf( psz_adev, sizeof(psz_adev), SDIAUDIO_DEVICE, p_sys->i_link );
-        psz_adev[sizeof(psz_adev) - 1] = '\0';
         if ( (p_sys->i_afd = vlc_open( psz_adev, O_RDONLY ) ) < 0 )
         {
             msg_Err( p_demux, "couldn't open device %s", psz_adev );
@@ -772,7 +789,7 @@ static int InitCapture( demux_t *p_demux )
             msg_Err( p_demux, "unknown sample rate %u", i_rate );
             return VLC_EGENERIC;
         }
-        close( p_sys->i_afd );
+        vlc_close( p_sys->i_afd );
 
         if ( InitAudio( p_demux ) != VLC_SUCCESS )
             return VLC_EGENERIC;
@@ -908,7 +925,7 @@ static void CloseCapture( demux_t *p_demux )
         munmap( p_sys->pp_vbuffers[i], p_sys->i_vbuffer_size );
     free( p_sys->pp_vbuffers );
 #endif
-    close( p_sys->i_vfd );
+    vlc_close( p_sys->i_vfd );
     if ( p_sys->i_max_channel != -1 )
     {
 #ifdef HAVE_MMAP_SDIAUDIO
@@ -916,30 +933,32 @@ static void CloseCapture( demux_t *p_demux )
             munmap( p_sys->pp_abuffers[i], p_sys->i_abuffer_size );
         free( p_sys->pp_abuffers );
 #endif
-        close( p_sys->i_afd );
+        vlc_close( p_sys->i_afd );
     }
 }
 
 static int Capture( demux_t *p_demux )
 {
     demux_sys_t *p_sys = p_demux->p_sys;
-    struct pollfd pfd[2];
+    struct pollfd pfd[3];
 
-    pfd[0].fd = p_sys->i_vfd;
-    pfd[0].events = POLLIN | POLLPRI;
+    pfd[0].fd = p_sys->evfd;
+    pfd[0].events = POLLIN;
+    pfd[1].fd = p_sys->i_vfd;
+    pfd[1].events = POLLIN | POLLPRI;
     if ( p_sys->i_max_channel != -1 )
     {
-        pfd[1].fd = p_sys->i_afd;
-        pfd[1].events = POLLIN | POLLPRI;
+        pfd[2].fd = p_sys->i_afd;
+        pfd[2].events = POLLIN | POLLPRI;
     }
 
-    if ( poll( pfd, 1 + (p_sys->i_max_channel != -1), READ_TIMEOUT ) < 0 )
-    {
-        msg_Warn( p_demux, "couldn't poll(): %s", vlc_strerror_c(errno) );
-        return VLC_EGENERIC;
-    }
+    if( poll( pfd, 2 + (p_sys->i_max_channel != -1), -1 ) < 0 )
+        return VLC_SUCCESS;
 
-    if ( pfd[0].revents & POLLPRI )
+    if( pfd[0].revents )
+        return VLC_EGENERIC; /* Stop! */
+
+    if( pfd[1].revents & POLLPRI )
     {
         unsigned int i_val;
 
@@ -964,7 +983,7 @@ static int Capture( demux_t *p_demux )
         p_sys->i_next_vdate += CLOCK_GAP;
     }
 
-    if ( p_sys->i_max_channel != -1 && pfd[1].revents & POLLPRI )
+    if( p_sys->i_max_channel != -1 && (pfd[2].revents & POLLPRI) )
     {
         unsigned int i_val;
 
@@ -987,7 +1006,7 @@ static int Capture( demux_t *p_demux )
         p_sys->i_next_vdate += CLOCK_GAP;
     }
 
-    if ( pfd[0].revents & POLLIN )
+    if( pfd[1].revents & POLLIN )
     {
 #ifdef HAVE_MMAP_SDIVIDEO
         if ( ioctl( p_sys->i_vfd, SDIVIDEO_IOC_DQBUF, p_sys->i_current_vbuffer )
@@ -1032,7 +1051,7 @@ static int Capture( demux_t *p_demux )
 #endif
     }
 
-    if ( p_sys->i_max_channel != -1 && pfd[1].revents & POLLIN )
+    if( p_sys->i_max_channel != -1 && (pfd[2].revents & POLLIN) )
     {
 #ifdef HAVE_MMAP_SDIAUDIO
         if ( ioctl( p_sys->i_afd, SDIAUDIO_IOC_DQBUF, p_sys->i_current_abuffer )
