@@ -33,6 +33,7 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <limits.h>
 
 #include <vlc_common.h>
 #include <vlc_plugin.h>
@@ -59,7 +60,7 @@
 
 #define DEMUX_BUFFER_SIZE 1350000
 #define MAX_AUDIOS        4
-#define SAMPLERATE_TOLERANCE 0.1
+#define SAMPLERATE_TOLERANCE 0.1f
 
 /*****************************************************************************
  * Module descriptor
@@ -859,7 +860,7 @@ static int InitAudio( demux_t *p_demux, sdi_audio_t *p_audio )
     p_audio->i_nb_samples         = p_audio->i_rate * p_sys->i_frame_rate_base
                                     / p_sys->i_frame_rate;
     p_audio->i_max_samples        = (float)p_audio->i_nb_samples *
-                                    (1. + SAMPLERATE_TOLERANCE);
+                                    (1.f + SAMPLERATE_TOLERANCE);
 
     p_audio->p_buffer             = malloc( p_audio->i_max_samples * sizeof(int16_t) * 2 );
     p_audio->i_left_samples       = p_audio->i_right_samples = 0;
@@ -875,7 +876,7 @@ static void ResampleAudio( int16_t *p_out, int16_t *p_in,
                            unsigned int i_out, unsigned int i_in )
 {
     unsigned int i_remainder = 0;
-    float f_last_sample = (float)*p_in / 32768.0;
+    float f_last_sample = (float)*p_in / 32768.f;
 
     *p_out = *p_in;
     p_out += 2;
@@ -883,14 +884,14 @@ static void ResampleAudio( int16_t *p_out, int16_t *p_in,
 
     for ( unsigned int i = 1; i < i_in; i++ )
     {
-        float f_in = (float)*p_in / 32768.0;
+        float f_in = (float)*p_in / 32768.f;
         while ( i_remainder < i_out )
         {
             float f_out = f_last_sample;
             f_out += (f_in - f_last_sample) * i_remainder / i_out;
-            if ( f_out >= 1.0 ) *p_out = 32767;
-            else if ( f_out < -1.0 ) *p_out = -32768;
-            else *p_out = f_out * 32768.0;
+            if ( f_out >= 1.f ) *p_out = 32767;
+            else if ( f_out < -1.f ) *p_out = -32768;
+            else *p_out = f_out * 32768.f;
             p_out += 2;
             i_remainder += i_in;
         }
@@ -916,9 +917,9 @@ static int DecodeAudio( demux_t *p_demux, sdi_audio_t *p_audio )
         return VLC_EGENERIC;
     }
     if ( p_audio->i_left_samples <
-            (float)p_audio->i_nb_samples * (1. - SAMPLERATE_TOLERANCE) ||
+            (float)p_audio->i_nb_samples * (1.f - SAMPLERATE_TOLERANCE) ||
         p_audio->i_left_samples >
-            (float)p_audio->i_nb_samples * (1. + SAMPLERATE_TOLERANCE) )
+            (float)p_audio->i_nb_samples * (1.f + SAMPLERATE_TOLERANCE) )
     {
         msg_Warn( p_demux,
             "left samplerate out of tolerance for audio %u/%u (%u vs. %u)",
@@ -927,9 +928,9 @@ static int DecodeAudio( demux_t *p_demux, sdi_audio_t *p_audio )
         return VLC_EGENERIC;
     }
     if ( p_audio->i_right_samples <
-            (float)p_audio->i_nb_samples * (1. - SAMPLERATE_TOLERANCE) ||
+            (float)p_audio->i_nb_samples * (1.f - SAMPLERATE_TOLERANCE) ||
         p_audio->i_right_samples >
-            (float)p_audio->i_nb_samples * (1. + SAMPLERATE_TOLERANCE) )
+            (float)p_audio->i_nb_samples * (1.f + SAMPLERATE_TOLERANCE) )
     {
         msg_Warn( p_demux,
             "right samplerate out of tolerance for audio %u/%u (%u vs. %u)",
@@ -1660,47 +1661,35 @@ static int HandleSDBuffer( demux_t *p_demux, uint8_t *p_buffer,
 
 static int ReadULSysfs( const char *psz_fmt, unsigned int i_link )
 {
-    char psz_file[MAXLEN], psz_data[MAXLEN];
-    char *psz_tmp;
-    int i_fd;
-    ssize_t i_ret;
+    char psz_file[MAXLEN];
     unsigned int i_data;
 
-    snprintf( psz_file, sizeof(psz_file) - 1, psz_fmt, i_link );
+    snprintf( psz_file, sizeof(psz_file), psz_fmt, i_link );
 
-    if ( (i_fd = vlc_open( psz_file, O_RDONLY )) < 0 )
-        return i_fd;
-
-    i_ret = read( i_fd, psz_data, sizeof(psz_data) );
-    close( i_fd );
-
-    if ( i_ret < 0 )
-        return i_ret;
-
-    i_data = strtoul( psz_data, &psz_tmp, 0 );
-    if ( *psz_tmp != '\n' )
+    FILE *stream = vlc_fopen( psz_file, "rt" );
+    if( stream == NULL )
         return -1;
 
-    return i_data;
+    int ret = fscanf( stream, "%u", &i_data );
+    fclose( stream );
+
+    return (ret == 1 && i_data <= INT_MAX) ? (int)i_data : -1;
 }
 
-static ssize_t WriteULSysfs( const char *psz_fmt, unsigned int i_link,
-                             unsigned int i_buf )
+static int WriteULSysfs( const char *psz_fmt, unsigned int i_link,
+                         unsigned int i_buf )
 {
-    char psz_file[MAXLEN], psz_data[MAXLEN];
-    int i_fd;
-    ssize_t i_ret;
+    char psz_file[MAXLEN];
 
-    snprintf( psz_file, sizeof(psz_file) -1, psz_fmt, i_link );
+    snprintf( psz_file, sizeof(psz_file), psz_fmt, i_link );
 
-    snprintf( psz_data, sizeof(psz_data) -1, "%u\n", i_buf );
+    FILE *stream = vlc_fopen( psz_file, "wt" );
+    if( stream == NULL )
+        return -1;
 
-    if ( (i_fd = vlc_open( psz_file, O_WRONLY )) < 0 )
-        return i_fd;
-
-    i_ret = write( i_fd, psz_data, strlen(psz_data) + 1 );
-    close( i_fd );
-    return i_ret;
+    int ret = fprintf( stream, "%u\n", i_buf );
+    fclose( stream );
+    return ret;
 }
 
 static int InitCapture( demux_t *p_demux )
@@ -1776,7 +1765,7 @@ static void CloseCapture( demux_t *p_demux )
     StopDecode( p_demux );
     for ( unsigned int i = 0; i < p_sys->i_buffers; i++ )
         munmap( p_sys->pp_buffers[i], p_sys->i_buffer_size );
-    close( p_sys->i_fd );
+    vlc_close( p_sys->i_fd );
     free( p_sys->pp_buffers );
 }
 
