@@ -1,24 +1,24 @@
 /*****************************************************************************
  * mms.c: MMS over tcp, udp and http access plug-in
  *****************************************************************************
- * Copyright (C) 2002-2004 the VideoLAN team
- * $Id: f01c1ed694cf69cd803ffc7c3d8a025464aaca8f $
+ * Copyright (C) 2002-2004 VLC authors and VideoLAN
+ * $Id: a6a0179e0f6fa421eae411d3acdcad1ddfcfe3b0 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -80,7 +80,7 @@ vlc_module_begin ()
     add_string( "mmsh-proxy", NULL, PROXY_TEXT, PROXY_LONGTEXT,
                     false )
 
-    add_shortcut( "mms", "mmsu", "mmst", "mmsh", "http" )
+    add_shortcut( "mms", "mmsu", "mmst", "mmsh" )
     set_callbacks( Open, Close )
 vlc_module_end ()
 
@@ -100,29 +100,15 @@ static int Open( vlc_object_t *p_this )
     access_t *p_access = (access_t*)p_this;
 
     /* use specified method */
-    if( *p_access->psz_access )
-    {
-        if( !strncmp( p_access->psz_access, "mmsu", 4 ) )
-        {
-            return  MMSTUOpen ( p_access );
-        }
-        else if( !strncmp( p_access->psz_access, "mmst", 4 ) )
-        {
-            return  MMSTUOpen ( p_access );
-        }
-        else if( !strncmp( p_access->psz_access, "mmsh", 4 ) ||
-                 !strncmp( p_access->psz_access, "http", 4 ) )
-        {
-            return  MMSHOpen ( p_access );
-        }
-    }
+    if( !strncmp( p_access->psz_access, "mmsu", 4 ) )
+        return  MMSTUOpen ( p_access );
+    else if( !strncmp( p_access->psz_access, "mmst", 4 ) )
+        return  MMSTUOpen ( p_access );
+    else if( !strncmp( p_access->psz_access, "mmsh", 4 ) )
+        return  MMSHOpen ( p_access );
 
     if( MMSTUOpen ( p_access ) )
-    {
-        if( p_access->b_die )
-            return VLC_EGENERIC;
-
-        /* try mmsh if mmstu failed */
+    {   /* try mmsh if mmstu failed */
         return  MMSHOpen ( p_access );
     }
     return VLC_SUCCESS;
