@@ -2,7 +2,7 @@
  * oss.c : OSS input module for vlc
  *****************************************************************************
  * Copyright (C) 2002-2009 VLC authors and VideoLAN
- * $Id: 40e78c1a09e733ec4ce204d19094de99d5512e12 $
+ * $Id: 04a0bd32ef46fe1777816b804f43277f059a02f2 $
  *
  * Authors: Benjamin Pracht <bigben at videolan dot org>
  *          Richard Hosking <richard at hovis dot net>
@@ -192,7 +192,8 @@ static void DemuxClose( vlc_object_t *p_this )
     demux_t     *p_demux = (demux_t *)p_this;
     demux_sys_t *p_sys   = p_demux->p_sys;
 
-    if( p_sys->i_fd >= 0 ) close( p_sys->i_fd );
+    if( p_sys->i_fd >= 0 )
+        vlc_close( p_sys->i_fd );
 
     if( p_sys->p_block ) block_Release( p_sys->p_block );
     free( p_sys );
@@ -376,7 +377,8 @@ static int OpenAudioDevOss( demux_t *p_demux )
 
  adev_fail:
 
-    if( i_fd >= 0 ) close( i_fd );
+    if( i_fd >= 0 )
+        vlc_close( i_fd );
     return -1;
 }
 
@@ -432,11 +434,13 @@ static bool ProbeAudioDevOss( demux_t *p_demux, const char *psz_device )
         goto open_failed;
     }
 
-    if( i_fd >= 0 ) close( i_fd );
+    if( i_fd >= 0 )
+        vlc_close( i_fd );
 
     return true;
 
 open_failed:
-    if( i_fd >= 0 ) close( i_fd );
+    if( i_fd >= 0 )
+        vöc_close( i_fd );
     return false;
 }
