@@ -2,7 +2,7 @@
  * access.c: Real rtsp input
  *****************************************************************************
  * Copyright (C) 2005 VideoLAN
- * $Id: e8b0ceca98d336fefb1ad2c5e5de3e3221452569 $
+ * $Id: d0eb69981912b4785a19e90e1b3533af17f59ff4 $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
@@ -28,6 +28,7 @@
 # include "config.h"
 #endif
 
+#define VLC_MODULE_LICENSE VLC_LICENSE_GPL_2_PLUS
 #include <vlc_common.h>
 #include <vlc_plugin.h>
 #include <vlc_access.h>
@@ -83,8 +84,8 @@ static int RtspConnect( void *p_userdata, char *psz_server, int i_port )
     if( p_sys->fd < 0 )
     {
         msg_Err( p_access, "cannot connect to %s:%d", psz_server, i_port );
-        dialog_Fatal( p_access, _("Connection failed"),
-                        _("VLC could not connect to \"%s:%d\"."), psz_server, i_port );
+        vlc_dialog_display_error( p_access, _("Connection failed"),
+            _("VLC could not connect to \"%s:%d\"."), psz_server, i_port );
         return VLC_EGENERIC;
     }
 
@@ -105,7 +106,7 @@ static int RtspRead( void *p_userdata, uint8_t *p_buffer, int i_buffer )
     access_t *p_access = (access_t *)p_userdata;
     access_sys_t *p_sys = p_access->p_sys;
 
-    return net_Read( p_access, p_sys->fd, 0, p_buffer, i_buffer, true );
+    return net_Read( p_access, p_sys->fd, p_buffer, i_buffer );
 }
 
 static int RtspReadLine( void *p_userdata, uint8_t *p_buffer, int i_buffer )
@@ -113,7 +114,7 @@ static int RtspReadLine( void *p_userdata, uint8_t *p_buffer, int i_buffer )
     access_t *p_access = (access_t *)p_userdata;
     access_sys_t *p_sys = p_access->p_sys;
 
-    char *psz = net_Gets( p_access, p_sys->fd, 0 );
+    char *psz = net_Gets( p_access, p_sys->fd );
 
     //fprintf(stderr, "ReadLine: %s\n", psz);
 
@@ -132,7 +133,7 @@ static int RtspWrite( void *p_userdata, uint8_t *p_buffer, int i_buffer )
 
     //fprintf(stderr, "Write: %s", p_buffer);
 
-    net_Printf( p_access, p_sys->fd, 0, "%s", p_buffer );
+    net_Printf( p_access, p_sys->fd, "%s", p_buffer );
 
     return 0;
 }
@@ -147,13 +148,8 @@ static int Open( vlc_object_t *p_this )
     char* psz_server = NULL;
     int i_result;
 
-    if( !p_access->psz_access || (
-        strncmp( p_access->psz_access, "rtsp", 4 ) &&
-        strncmp( p_access->psz_access, "pnm", 3 )  &&
-        strncmp( p_access->psz_access, "realrtsp", 8 ) ))
-    {
-            return VLC_EGENERIC;
-    }
+    if( p_access->b_preparsing )
+        return VLC_EGENERIC;
 
     /* Discard legacy username/password syntax - not supported */
     const char *psz_location = strchr( p_access->psz_location, '@' );
@@ -166,7 +162,6 @@ static int Open( vlc_object_t *p_this )
     p_access->pf_block = BlockRead;
     p_access->pf_seek = Seek;
     p_access->pf_control = Control;
-    p_access->info.i_pos = 0;
     p_access->info.b_eof = false;
     p_access->p_sys = p_sys = malloc( sizeof( access_sys_t ) );
     if( !p_sys )
@@ -228,8 +223,8 @@ static int Open( vlc_object_t *p_this )
 
 
             msg_Err( p_access, "rtsp session can not be established" );
-            dialog_Fatal( p_access, _("Session failed"), "%s",
-                    _("The requested RTSP session could not be established.") );
+            vlc_dialog_display_error( p_access, _("Session failed"), "%s",
+                _("The requested RTSP session could not be established.") );
             goto error;
         }
 
