@@ -2,7 +2,7 @@
  * screen.c: Screen capture module.
  *****************************************************************************
  * Copyright (C) 2004-2008 VLC authors and VideoLAN
- * $Id: 862dbb42cae2a1b7ddeea2a203cea8c4ce7dc8ca $
+ * $Id: 66a1396228700b0759312fd0689db2e5461eab0e $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *          Antoine Cellerier <dionoea at videolan dot org>
@@ -33,6 +33,7 @@
 #include <vlc_common.h>
 #include <vlc_plugin.h>
 #include <vlc_modules.h>                 /* module_need for "video blending" */
+#include <vlc_filter.h>
 #include <vlc_url.h>
 #include "screen.h"
 
@@ -51,17 +52,17 @@
 #endif
 
 #ifdef SCREEN_SUBSCREEN
-#define TOP_TEXT N_( "Subscreen top left corner" )
+#define TOP_TEXT N_( "Region top row" )
 #define TOP_LONGTEXT N_( \
-    "Top coordinate of the subscreen top left corner." )
+    "Ordinate of the capture region in pixels." )
 
-#define LEFT_TEXT N_( "Subscreen top left corner" )
+#define LEFT_TEXT N_( "Region left column" )
 #define LEFT_LONGTEXT N_( \
-    "Left coordinate of the subscreen top left corner." )
+    "Abscissa of the capture region in pixels." )
 
-#define WIDTH_TEXT N_( "Subscreen width" )
+#define WIDTH_TEXT N_( "Capture region width" )
 
-#define HEIGHT_TEXT N_( "Subscreen height" )
+#define HEIGHT_TEXT N_( "Capture region heigh" )
 
 #define FOLLOW_MOUSE_TEXT N_( "Follow the mouse" )
 #define FOLLOW_MOUSE_LONGTEXT N_( \
@@ -179,9 +180,6 @@ static int Open( vlc_object_t *p_this )
         return VLC_EGENERIC;
     }
 
-    p_sys->fmt.video.i_frame_rate      = 1000000;
-    p_sys->fmt.video.i_frame_rate_base = p_sys->i_incr;
-
     msg_Dbg( p_demux, "screen width: %i, height: %i, depth: %i",
              p_sys->fmt.video.i_width, p_sys->fmt.video.i_height,
              p_sys->fmt.video.i_bits_per_pixel );
