@@ -2,9 +2,10 @@
  * sftp.c: SFTP input module
  *****************************************************************************
  * Copyright (C) 2009 VLC authors and VideoLAN
- * $Id: d8d132d0b6c868b13e072df8fb9eb5bd1f74a650 $
+ * $Id: 498626b27c0e1a0f31566b860e65d44212f041c7 $
  *
  * Authors: Rémi Duraffort <ivoire@videolan.org>
+ *          Petri Hintukainen <phintuka@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published by
@@ -34,9 +35,10 @@
 #include <assert.h>
 
 #include <vlc_access.h>
-#include <vlc_dialog.h>
+#include <vlc_input_item.h>
 #include <vlc_network.h>
 #include <vlc_url.h>
+#include <vlc_keystore.h>
 
 #include <libssh2.h>
 #include <libssh2_sftp.h>
@@ -50,8 +52,12 @@ static void Close( vlc_object_t* );
 
 #define PORT_TEXT N_("SFTP port")
 #define PORT_LONGTEXT N_("SFTP port number to use on the server")
-#define MTU_TEXT N_("Read size")
-#define MTU_LONGTEXT N_("Size of the request for reading access")
+#define USER_TEXT N_("Username")
+#define USER_LONGTEXT N_("Username that will be used for the connection, " \
+        "if no username is set in the URL.")
+#define PASS_TEXT N_("Password")
+#define PASS_LONGTEXT N_("Password that will be used for the connection, " \
+        "if no username or password are set in URL.")
 
 vlc_module_begin ()
     set_shortname( "SFTP" )
@@ -59,8 +65,9 @@ vlc_module_begin ()
     set_capability( "access", 0 )
     set_category( CAT_INPUT )
     set_subcategory( SUBCAT_INPUT_ACCESS )
-    add_integer( "sftp-readsize", 8192, MTU_TEXT, MTU_LONGTEXT, true )
     add_integer( "sftp-port", 22, PORT_TEXT, PORT_LONGTEXT, true )
+    add_string( "sftp-user", NULL, USER_TEXT, USER_LONGTEXT, false )
+    add_password( "sftp-pwd", NULL, PASS_TEXT, PASS_LONGTEXT, false )
     add_shortcut( "sftp" )
     set_callbacks( Open, Close )
 vlc_module_end ()
@@ -69,10 +76,12 @@ vlc_module_end ()
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
-static block_t* Block( access_t * );
+static ssize_t  Read( access_t *, uint8_t *, size_t );
 static int      Seek( access_t *, uint64_t );
 static int      Control( access_t *, int, va_list );
 
+static int DirRead( access_t *, input_item_node_t * );
+static int DirControl( access_t *, int, va_list );
 
 struct access_sys_t
 {
@@ -81,10 +90,38 @@ struct access_sys_t
     LIBSSH2_SFTP* sftp_session;
     LIBSSH2_SFTP_HANDLE* file;
     uint64_t filesize;
-    size_t i_read_size;
+    char *psz_base_url;
 };
 
 
+static int AuthPublicKey( access_t *p_access, const char *psz_home, const char *psz_username )
+{
+    access_sys_t* p_sys = p_access->p_sys;
+    int i_result = VLC_EGENERIC;
+    char *psz_keyfile1 = NULL;
+    char *psz_keyfile2 = NULL;
+
+    if( !psz_username || !psz_username[0] )
+        return i_result;
+
+    if( asprintf( &psz_keyfile1, "%s/.ssh/id_rsa.pub", psz_home ) == -1 ||
+        asprintf( &psz_keyfile2, "%s/.ssh/id_rsa",     psz_home ) == -1 )
+        goto bailout;
+
+    if( libssh2_userauth_publickey_fromfile( p_sys->ssh_session, psz_username, psz_keyfile1, psz_keyfile2, NULL ) )
+    {
+        msg_Dbg( p_access, "Public key authentication failed" );
+        goto bailout;
+    }
+
+    msg_Info( p_access, "Public key authentication succeeded" );
+    i_result = VLC_SUCCESS;
+
+ bailout:
+    free( psz_keyfile1 );
+    free( psz_keyfile2 );
+    return i_result;
+}
 
 /**
  * Connect to the sftp server and ask for a file
@@ -95,23 +132,33 @@ static int Open( vlc_object_t* p_this )
 {
     access_t*   p_access = (access_t*)p_this;
     access_sys_t* p_sys;
-    char* psz_username = NULL;
-    char* psz_password = NULL;
+    vlc_url_t credential_url;
+    vlc_credential credential;
+    const char* psz_path;
+    char* psz_remote_home = NULL;
+    char* psz_home = NULL;
     int i_port;
     int i_ret;
     vlc_url_t url;
     size_t i_len;
     int i_type;
+    int i_result = VLC_EGENERIC;
 
     if( !p_access->psz_location )
         return VLC_EGENERIC;
 
-    STANDARD_BLOCK_ACCESS_INIT;
+    access_InitFields( p_access );
+    p_sys = p_access->p_sys = (access_sys_t*)calloc( 1, sizeof( access_sys_t ) );
+    if( !p_sys ) return VLC_ENOMEM;
+
     p_sys->i_socket = -1;
 
+    vlc_UrlParse( &credential_url, p_access->psz_url );
+    vlc_credential_init( &credential, &credential_url );
+
     /* Parse the URL */
-    const char* path = p_access->psz_location;
-    vlc_UrlParse( &url, path, 0 );
+    vlc_UrlParse( &url, p_access->psz_location );
+    vlc_uri_decode( url.psz_path );
 
     /* Check for some parameters */
     if( EMPTY_STR( url.psz_host ) )
@@ -120,22 +167,6 @@ static int Open( vlc_object_t* p_this )
         goto error;
     }
 
-    /* If the user name is empty, ask the user */
-    if( !EMPTY_STR( url.psz_username ) && url.psz_password )
-    {
-        psz_username = strdup( url.psz_username );
-        psz_password = strdup( url.psz_password );
-    }
-    else
-    {
-        dialog_Login( p_access, &psz_username, &psz_password,
-                      _("SFTP authentication"),
-                      _("Please enter a valid login and password for the sftp "
-                        "connexion to %s"), url.psz_host );
-        if( EMPTY_STR(psz_username) || !psz_password )
-            goto error;
-    }
-
     if( url.i_port <= 0 )
         i_port = var_InheritInteger( p_access, "sftp-port" );
     else
@@ -172,7 +203,7 @@ static int Open( vlc_object_t* p_this )
     if( !ssh_knownhosts )
         goto error;
 
-    char *psz_home = config_GetUserDir( VLC_HOME_DIR );
+    psz_home = config_GetUserDir( VLC_HOME_DIR );
     char *psz_knownhosts_file;
     if( asprintf( &psz_knownhosts_file, "%s/.ssh/known_hosts", psz_home ) != -1 )
     {
@@ -180,7 +211,6 @@ static int Open( vlc_object_t* p_this )
                 LIBSSH2_KNOWNHOST_FILE_OPENSSH );
         free( psz_knownhosts_file );
     }
-    free( psz_home );
 
     const char *fingerprint = libssh2_session_hostkey( p_sys->ssh_session, &i_len, &i_type );
     struct libssh2_knownhost *host;
@@ -209,11 +239,28 @@ static int Open( vlc_object_t* p_this )
 
     //TODO: ask for the available auth methods
 
-    /* send the login/password */
-    if( libssh2_userauth_password( p_sys->ssh_session, psz_username, psz_password ) )
+    /* Try public key auth first */
+    if( AuthPublicKey( p_access, psz_home, url.psz_username ) != VLC_SUCCESS )
     {
-        msg_Err( p_access, "Authentication by password failed" );
-        goto error;
+    while( vlc_credential_get( &credential, p_access, "sftp-user", "sftp-pwd",
+                               _("SFTP authentication"),
+                               _("Please enter a valid login and password for "
+                               "the sftp connexion to %s"), url.psz_host ) )
+    {
+        /* send the login/password */
+        if( libssh2_userauth_password( p_sys->ssh_session,
+                                       credential.psz_username,
+                                       credential.psz_password ) == 0 )
+        {
+            vlc_credential_store( &credential, p_access );
+            break;
+        }
+
+        if( AuthPublicKey( p_access, psz_home, credential.psz_username ) == VLC_SUCCESS )
+            break;
+
+        msg_Warn( p_access, "sftp auth failed for %s", credential.psz_username );
+    }
     }
 
     /* Create the sftp session */
@@ -225,42 +272,95 @@ static int Open( vlc_object_t* p_this )
         goto error;
     }
 
-    /* Open the given file */
-    p_sys->file = libssh2_sftp_open( p_sys->sftp_session, url.psz_path, LIBSSH2_FXF_READ, 0 );
-    if( !p_sys->file )
+    /* No path, default to user Home */
+    if( !url.psz_path )
     {
-        msg_Err( p_access, "Unable to open the remote file %s", url.psz_path );
-        goto error;
+        const size_t i_size = 1024;
+        int i_ret;
+
+        psz_remote_home = malloc( i_size );
+        if( !psz_remote_home )
+            goto error;
+
+        i_ret = libssh2_sftp_symlink_ex( p_sys->sftp_session, ".", 1,
+                                         psz_remote_home, i_size - 1,
+                                         LIBSSH2_SFTP_REALPATH );
+        if( i_ret <= 0 )
+        {
+            msg_Err( p_access, "Impossible to get the Home directory" );
+            goto error;
+        }
+        psz_remote_home[i_ret] = '\0';
+        psz_path = psz_remote_home;
+
+        /* store base url for directory read */
+        char *base = vlc_path2uri( psz_path, "sftp" );
+        if( !base )
+            goto error;
+        if( -1 == asprintf( &p_sys->psz_base_url, "sftp://%s%s", p_access->psz_location, base + 7 ) )
+        {
+            free( base );
+            goto error;
+        }
+        free( base );
     }
+    else
+        psz_path = url.psz_path;
 
     /* Get some information */
     LIBSSH2_SFTP_ATTRIBUTES attributes;
-    if( libssh2_sftp_stat( p_sys->sftp_session, url.psz_path, &attributes ) )
+    if( libssh2_sftp_stat( p_sys->sftp_session, psz_path, &attributes ) )
     {
-        msg_Err( p_access, "Impossible to get information about the remote file %s", url.psz_path );
+        msg_Err( p_access, "Impossible to get information about the remote path %s", psz_path );
         goto error;
     }
-    p_sys->filesize = attributes.filesize;
 
-    p_sys->i_read_size = var_InheritInteger( p_access, "sftp-readsize" );
+    if( !LIBSSH2_SFTP_S_ISDIR( attributes.permissions ))
+    {
+        /* Open the given file */
+        p_sys->file = libssh2_sftp_open( p_sys->sftp_session, psz_path, LIBSSH2_FXF_READ, 0 );
+        p_sys->filesize = attributes.filesize;
 
-    free( psz_password );
-    free( psz_username );
-    vlc_UrlClean( &url );
-    return VLC_SUCCESS;
+        ACCESS_SET_CALLBACKS( Read, NULL, Control, Seek );
+    }
+    else
+    {
+        /* Open the given directory */
+        p_sys->file = libssh2_sftp_opendir( p_sys->sftp_session, psz_path );
+
+        p_access->pf_readdir = DirRead;
+        p_access->pf_control = DirControl;
+
+        if( !p_sys->psz_base_url )
+        {
+            if( asprintf( &p_sys->psz_base_url, "sftp://%s", p_access->psz_location ) == -1 )
+                goto error;
+
+            /* trim trailing '/' */
+            size_t len = strlen( p_sys->psz_base_url );
+            if( len > 0 && p_sys->psz_base_url[ len - 1 ] == '/' )
+                p_sys->psz_base_url[ len - 1 ] = 0;
+        }
+    }
+
+    if( !p_sys->file )
+    {
+        msg_Err( p_access, "Unable to open the remote path %s", psz_path );
+        goto error;
+    }
+
+    i_result = VLC_SUCCESS;
 
 error:
-    if( p_sys->file )
-        libssh2_sftp_close_handle( p_sys->file );
-    if( p_sys->ssh_session )
-        libssh2_session_free( p_sys->ssh_session );
-    free( psz_password );
-    free( psz_username );
+    free( psz_home );
+    free( psz_remote_home );
     vlc_UrlClean( &url );
-    if( p_sys->i_socket >= 0 )
-        net_Close( p_sys->i_socket );
-    free( p_sys );
-    return VLC_EGENERIC;
+    vlc_credential_clean( &credential );
+    vlc_UrlClean( &credential_url );
+    if( i_result != VLC_SUCCESS ) {
+        Close( p_this );
+    }
+    return i_result;
 }
 
 
@@ -270,56 +370,43 @@ static void Close( vlc_object_t* p_this )
     access_t*   p_access = (access_t*)p_this;
     access_sys_t* p_sys = p_access->p_sys;
 
-    libssh2_sftp_close_handle( p_sys->file );
-    libssh2_sftp_shutdown( p_sys->sftp_session );
+    if( p_sys->file )
+        libssh2_sftp_close_handle( p_sys->file );
+    if( p_sys->sftp_session )
+        libssh2_sftp_shutdown( p_sys->sftp_session );
+    if( p_sys->ssh_session )
+        libssh2_session_free( p_sys->ssh_session );
+    if( p_sys->i_socket >= 0 )
+        net_Close( p_sys->i_socket );
 
-    libssh2_session_free( p_sys->ssh_session );
-    net_Close( p_sys->i_socket );
+    free( p_sys->psz_base_url );
     free( p_sys );
 }
 
 
-static block_t* Block( access_t* p_access )
+static ssize_t Read( access_t *p_access, uint8_t *buf, size_t len )
 {
     access_sys_t *p_sys = p_access->p_sys;
 
     if( p_access->info.b_eof )
-        return NULL;
+        return 0;
 
-    /* Allocate the buffer we need */
-    size_t i_len = __MIN( p_sys->i_read_size,
-                          p_sys->filesize - p_access->info.i_pos );
-    block_t* p_block = block_Alloc( i_len );
-    if( !p_block )
-        return NULL;
-
-    /* Read the specified size */
-    ssize_t i_ret = libssh2_sftp_read( p_access->p_sys->file, (char*)p_block->p_buffer, i_len );
-
-    if( i_ret < 0 )
-    {
-        block_Release( p_block );
-        msg_Err( p_access, "read failed" );
-        return NULL;
-    }
-    else if( i_ret == 0 )
+    ssize_t val = libssh2_sftp_read(  p_sys->file, (char *)buf, len );
+    if( val < 0 )
     {
         p_access->info.b_eof = true;
-        block_Release( p_block );
-        return NULL;
-    }
-    else
-    {
-        p_block->i_buffer = i_ret;
-        p_access->info.i_pos += i_ret;
-        return p_block;
+        msg_Err( p_access, "read failed" );
+        return 0;
     }
+    else if( val == 0 )
+        p_access->info.b_eof = true;
+
+    return val;
 }
 
 
 static int Seek( access_t* p_access, uint64_t i_pos )
 {
-    p_access->info.i_pos = i_pos;
     p_access->info.b_eof = false;
 
     libssh2_sftp_seek( p_access->p_sys->file, i_pos );
@@ -351,6 +438,8 @@ static int Control( access_t* p_access, int i_query, va_list args )
         break;
 
     case ACCESS_GET_SIZE:
+        if( p_access->pf_readdir != NULL )
+            return VLC_EGENERIC;
         *va_arg( args, uint64_t * ) = p_access->p_sys->filesize;
         break;
 
@@ -370,3 +459,90 @@ static int Control( access_t* p_access, int i_query, va_list args )
     return VLC_SUCCESS;
 }
 
+
+/*****************************************************************************
+ * Directory access
+ *****************************************************************************/
+
+static int DirRead (access_t *p_access, input_item_node_t *p_current_node)
+{
+    access_sys_t *p_sys = p_access->p_sys;
+    LIBSSH2_SFTP_ATTRIBUTES attrs;
+    int i_ret = VLC_SUCCESS;
+    int err;
+    /* Allocate 1024 bytes for file name. Longer names are skipped.
+     * libssh2 does not support seeking in directory streams.
+     * Retrying with larger buffer is possible, but would require
+     * re-opening the directory stream.
+     */
+    size_t i_size = 1024;
+    char *psz_file = malloc( i_size );
+
+    if( !psz_file )
+        return VLC_ENOMEM;
+
+    struct access_fsdir fsdir;
+    access_fsdir_init( &fsdir, p_access, p_current_node );
+
+    while( i_ret == VLC_SUCCESS
+        && 0 != ( err = libssh2_sftp_readdir( p_sys->file, psz_file, i_size, &attrs ) ) )
+    {
+        if( err < 0 )
+        {
+            if( err == LIBSSH2_ERROR_BUFFER_TOO_SMALL )
+            {
+                /* seeking back is not possible ... */
+                msg_Dbg( p_access, "skipped too long file name" );
+                continue;
+            }
+            if( err == LIBSSH2_ERROR_EAGAIN )
+            {
+                continue;
+            }
+            msg_Err( p_access, "directory read failed" );
+            break;
+        }
+
+        /* Create an input item for the current entry */
+
+        char *psz_full_uri, *psz_uri;
+
+        psz_uri = vlc_uri_encode( psz_file );
+        if( psz_uri == NULL )
+        {
+            i_ret = VLC_ENOMEM;
+            break;
+        }
+
+        if( asprintf( &psz_full_uri, "%s/%s", p_sys->psz_base_url, psz_uri ) == -1 )
+        {
+            free( psz_uri );
+            i_ret = VLC_ENOMEM;
+            break;
+        }
+        free( psz_uri );
+
+        int i_type = LIBSSH2_SFTP_S_ISDIR( attrs.permissions ) ? ITEM_TYPE_DIRECTORY : ITEM_TYPE_FILE;
+        i_ret = access_fsdir_additem( &fsdir, psz_full_uri, psz_file, i_type,
+                                      ITEM_NET );
+        free( psz_full_uri );
+    }
+
+    access_fsdir_finish( &fsdir, i_ret == VLC_SUCCESS );
+    free( psz_file );
+    return i_ret;
+}
+
+static int DirControl( access_t *p_access, int i_query, va_list args )
+{
+    switch( i_query )
+    {
+    case ACCESS_IS_DIRECTORY:
+        *va_arg( args, bool * ) = true; /* might loop */
+        break;
+    default:
+        return access_vaDirectoryControlHelper( p_access, i_query, args );
+    }
+
+    return VLC_SUCCESS;
+}
