@@ -33,6 +33,7 @@
 
 #include <vlc_common.h>
 #include <vlc_access.h>
+#include <vlc_interrupt.h>
 
 #include "v4l2.h"
 
@@ -58,6 +59,9 @@ int AccessOpen( vlc_object_t *obj )
 {
     access_t *access = (access_t *)obj;
 
+    if( access->b_preparsing )
+        return VLC_EGENERIC;
+
     access_InitFields( access );
 
     access_sys_t *sys = calloc (1, sizeof (*sys));
@@ -123,7 +127,8 @@ int InitVideo (access_t *access, int fd, uint32_t caps)
         struct v4l2_format fmt = { .type = V4L2_BUF_TYPE_VIDEO_CAPTURE };
         if (v4l2_ioctl (fd, VIDIOC_G_FMT, &fmt) < 0)
         {
-            msg_Err (access, "cannot get default format: %m");
+            msg_Err (access, "cannot get default format: %s",
+                     vlc_strerror_c(errno));
             return -1;
         }
         pixfmt = fmt.fmt.pix.pixelformat;
@@ -203,18 +208,7 @@ static int AccessPoll (access_t *access)
     ufd.fd = sys->fd;
     ufd.events = POLLIN;
 
-    switch (poll (&ufd, 1, 500))
-    {
-        case -1:
-            if (errno == EINTR)
-        case 0:
-            /* FIXME: kill this case (arbitrary timeout) */
-                return -1;
-            msg_Err (access, "poll error: %m");
-            access->info.b_eof = true;
-            return -1;
-    }
-    return 0;
+    return vlc_poll_i11e (&ufd, 1, -1);
 }
 
 
@@ -249,13 +243,12 @@ static block_t *ReadBlock (access_t *access)
     if (val < 0)
     {
         block_Release (block);
-        msg_Err (access, "cannot read buffer: %m");
+        msg_Err (access, "cannot read buffer: %s", vlc_strerror_c(errno));
         access->info.b_eof = true;
         return NULL;
     }
 
     block->i_buffer = val;
-    access->info.i_pos += val;
     return block;
 }
 
@@ -263,7 +256,6 @@ static int AccessControl( access_t *access, int query, va_list args )
 {
     switch( query )
     {
-        /* */
         case ACCESS_CAN_SEEK:
         case ACCESS_CAN_FASTSEEK:
         case ACCESS_CAN_PAUSE:
@@ -271,27 +263,16 @@ static int AccessControl( access_t *access, int query, va_list args )
             *va_arg( args, bool* ) = false;
             break;
 
-        /* */
         case ACCESS_GET_PTS_DELAY:
             *va_arg(args,int64_t *) = INT64_C(1000)
                 * var_InheritInteger( access, "live-caching" );
             break;
 
-        /* */
         case ACCESS_SET_PAUSE_STATE:
             /* Nothing to do */
             break;
 
-        case ACCESS_GET_TITLE_INFO:
-        case ACCESS_SET_TITLE:
-        case ACCESS_SET_SEEKPOINT:
-        case ACCESS_SET_PRIVATE_ID_STATE:
-        case ACCESS_GET_CONTENT_TYPE:
-        case ACCESS_GET_META:
-            return VLC_EGENERIC;
-
         default:
-            msg_Warn( access, "Unimplemented query %d in control", query );
             return VLC_EGENERIC;
 
     }
